import os
from unittest import TestCase, mock

import pytest

from jovian._version import __version__
from jovian.tests.resources.shared import MockResponse, fake_creds
from jovian.utils import slack
from jovian.utils.credentials import write_creds
from jovian.utils.error import ApiError
from jovian.utils.slack import _h, add_slack, notify


def test_h():
    with fake_creds():
        expected_result = {
            "Authorization": "Bearer fake_api_key",
            "x-jovian-source": "library",
            "x-jovian-library-version": __version__,
            "x-jovian-command": "add-slack",
            "x-jovian-guest": "b6538d4dfde04fcf993463a828a9cec6",
            "x-jovian-org": "staging",
        }

        assert _h() == expected_result


def mock_requests_get(url, *args, **kwargs):
    if url == "https://api-staging.jovian.ai/slack/integration_details":
        if kwargs["headers"]["Authorization"] == "Bearer fake_api_key":
            data = {
                "data": {
                    "slackAccount": {
                        "accountId": 47,
                        "channel": "@rohit",
                        "deleted": False,
                        "id": 10,
                        "updatedAt": "Fri, 13 Dec 2019 13:41:27 GMT",
                        "workspace": "jovian.ml",
                    }
                },
                "success": True,
            }

            return MockResponse(data, status_code=200)

        elif kwargs["headers"]["Authorization"] == "Bearer fake_api_key_error":
            data = {
                "errors": [{"code": 200, "message": "Invalid guest key"}],
                "success": False,
            }
            return MockResponse(data, 200)

        else:
            data = {
                "errors": [{"code": 401, "message": "Missing Authorization Header"}],
                "success": False,
            }
            return MockResponse(data, 401)


@pytest.mark.parametrize(
    "api_key, expected_result",
    [
        (
            "fake_api_key",
            "[jovian] Slack already connected."
            + " \nWorkspace: jovian.ml\nConnected Channel: @rohit"
        ),
        (
            "fake_api_key_error",
            "[jovian] Invalid guest key"
        ),
    ]
)
@mock.patch("jovian.utils.slack.get", side_effect=mock_requests_get)
def test_add_slack(mock_requests_get, api_key, expected_result, capsys):
    with fake_creds():
        creds = {
            "WEBAPP_URL": "https://staging.jovian.ml/",
            "GUEST_KEY": "b6538d4dfde04fcf993463a828a9cec6",
            "API_URL": "https://api-staging.jovian.ai",
            "API_KEY": api_key,
            "ORG_ID": "staging",
        }
        write_creds(creds)

        add_slack()

        captured = capsys.readouterr()
        assert captured.out.strip() == expected_result


@mock.patch("jovian.utils.slack.get", side_effect=mock_requests_get)
def test_add_slack_api_error(mock_get):
    with fake_creds():
        creds = {
            "WEBAPP_URL": "https://staging.jovian.ml/",
            "GUEST_KEY": "b6538d4dfde04fcf993463a828a9cec6",
            "API_URL": "https://api-staging.jovian.ai",
            "API_KEY": "fake_invalid_api_key",
            "ORG_ID": "staging",
        }
        write_creds(creds)

        with pytest.raises(ApiError):
            add_slack()


@pytest.mark.parametrize(
    "resp, expected_result, safe",
    [
        (
            {
                "data": {
                    "messageId": "60023f08f3b54801bd58cf6b37067ed6",
                    "messageSent": True,
                },
                "success": True,
            },
            "[jovian] message_sent:True",
            False
        ),
        (
            {
                "errors": [{"code": 401, "message": "The token is invalid"}],
                "success": False,
            },
            "[jovian] Error: The token is invalid",
            True
        ),
        (
            {
                "errors": [{"code": 200, "message": "The token has expired"}],
                "success": False,
            },
            "[jovian] Error: The token has expired",
            False
        ),
    ]
)
def test_notify(resp, expected_result, safe, capsys):
    with mock.patch("jovian.utils.slack.post_slack_message", return_value=resp):
        notify({"key": "value"}, safe=safe)

        captured = capsys.readouterr()
        if "errors" in resp:
            assert captured.err.strip() == expected_result
        else:
            assert captured.out.strip() == expected_result



import os
from unittest import TestCase, mock

import pytest

from jovian.utils.extension import setup_extension


@pytest.mark.parametrize(
    "enable, expected_result",
    [
        (True, "jupyter nbextension enable jovian_nb_ext/main --sys-prefix"),
        (False, "jupyter nbextension disable jovian_nb_ext/main --sys-prefix"),
    ]
)
@mock.patch('os.system')
def test_setup_extension_true(mock_system, enable, expected_result):
    setup_extension(enable=enable)
    mock_system.assert_called_with(expected_result)



from tempfile import TemporaryDirectory
from textwrap import dedent
from unittest import TestCase, mock
from unittest.mock import ANY
import pytest

from jovian.tests.resources.shared import fake_creds, temp_directory, MockResponse
from jovian.utils.configure import configure, reset_config
from jovian.utils.credentials import creds_exist, purge_creds, read_creds


def mock_request_get(*args, **kwargs):
    return MockResponse({
        "CONFIG_NAME": "staging",
        "APP_NAME": "Jovian (Staging)",
        "APP_DESCRIPTION": "Share Jupyter notebooks instantly",
        "IS_PRODUCTION": True,
        "IS_FLAVOR_PRO": False,
        "COMPANY_LOGO_URL": "https://i.imgur.com/iOPEHyK.png",
        "API_URL": "https://api-staging.jovian.ai",
        "API_URL_ALT": "https://staging.jovian.ml/api",
        "AUTH_ENV": "staging",
        "LOGIN_REDIRECT_PATH": "/login",
    }, 200)


@mock.patch("click.confirm", return_value=True)
def test_reset_config_prompt_confirmation(mock_confirm, capsys):
    with fake_creds():
        reset_config()
        assert creds_exist() == False

        expected_result = "[jovian] Removing existing configuration. Run \"jovian configure\" to set up Jovian"
        captured = capsys.readouterr()
        assert captured.out.strip() == expected_result


def test_reset_config_no_prompt_confirmation(capsys):
    with fake_creds():
        reset_config(confirm=False)
        assert creds_exist() == False

        expected_result = "[jovian] Removing existing configuration. Run \"jovian configure\" to set up Jovian"
        captured = capsys.readouterr()
        assert captured.out.strip() == expected_result


@mock.patch("click.confirm", return_value=False)
@mock.patch("jovian.utils.configure.purge_creds")
def test_reset_config_confirm_false(mock_purge_creds, mock_confirm, capsys):
    with fake_creds():
        reset_config()
        mock_purge_creds.assert_not_called()

        expected_result = "[jovian] Skipping.."
        captured = capsys.readouterr()
        assert captured.out.strip() == expected_result


def test_reset_config_no_creds(capsys):
    with fake_creds():
        purge_creds()

        reset_config()

        expected_result = "[jovian] Jovian is not configured yet. Run \"jovian configure\" to set it up."
        captured = capsys.readouterr()
        assert captured.out.strip() == expected_result


@mock.patch("requests.get", side_effect=mock_request_get)
@mock.patch("jovian.utils.credentials.validate_api_key", return_value=True)
@mock.patch("click.prompt", side_effect=["staging", "fake_api_key"])
@mock.patch("click.confirm", return_value=True)
def test_configure_confirm_yes(mock_confirm, mock_prompt, mock_validate_api_key, mock_get, capsys):
    with fake_creds():
        configure()

        assert read_creds() == {'API_KEY': 'fake_api_key',
                                'API_URL': 'https://api-staging.jovian.ai',
                                'GUEST_KEY': ANY,
                                'ORG_ID': 'staging',
                                'WEBAPP_URL': 'https://staging.jovian.ml/'}

        expected_result = dedent("""
        [jovian] It looks like Jovian is already configured ( check ~/.jovian/credentials.json ).
        [jovian] Removing existing configuration..
        [jovian] If you're a jovian-pro user please enter your company's organization ID on Jovian (otherwise leave it blank).
        [jovian] Please enter your API key ( from https://staging.jovian.ml/ ):
        [jovian] Configuration complete!
        """).strip()

        captured = capsys.readouterr()
        assert captured.out.strip() == expected_result.strip()


@mock.patch("requests.get", side_effect=mock_request_get)
@mock.patch("jovian.utils.credentials.validate_api_key", return_value=True)
@mock.patch("click.prompt", side_effect=["staging", "fake_api_key"])
def test_configure_no_creds(mock_prompt, mock_validate_api_key, mock_get, capsys):
    with fake_creds():
        purge_creds()

        configure()

        assert read_creds() == {'API_KEY': 'fake_api_key',
                                'API_URL': 'https://api-staging.jovian.ai',
                                'GUEST_KEY': ANY,
                                'ORG_ID': 'staging',
                                'WEBAPP_URL': 'https://staging.jovian.ml/'}

        expected_result = dedent("""
        [jovian] If you're a jovian-pro user please enter your company's organization ID on Jovian (otherwise leave it blank).
        [jovian] Please enter your API key ( from https://staging.jovian.ml/ ):
        [jovian] Configuration complete!
        """).strip()

        captured = capsys.readouterr()
        assert captured.out.strip() == expected_result.strip()


@mock.patch("click.confirm", return_value=False)
def test_configure_confirm_no(mock_confirm, capsys):
    with fake_creds():
        creds = read_creds()
        configure()

        # Check that creds were not modified
        assert read_creds() == creds

        expected_result = dedent("""
        [jovian] It looks like Jovian is already configured ( check ~/.jovian/credentials.json ).
        [jovian] Skipping..
        """).strip()
        captured = capsys.readouterr()
        assert captured.out.strip() == expected_result.strip()



import pytest
from click.testing import CliRunner
from unittest import mock
from jovian.cli import main


@pytest.fixture(scope='module')
def runner():
    # Get instance of CliRunner
    runner = CliRunner()
    return runner


@pytest.mark.parametrize(
    "func, cli_args, called_with_args",
    [
        # enable-extension
        ("setup_extension", ["enable-extension"], {"enable": True}),

        # disable-extension
        ("setup_extension", ["disable-extension"], {"enable": False}),

        # add-slack
        ("add_slack", ["add-slack"], {}),

        # activate
        ("activate", ["activate"], {}),

        # reset
        ("reset_config", ["reset"], {}),

        # configure
        ("configure", ["configure"], {}),

        # install
        ("install", ["install"], {}),
        ("install", ['install', '-n', 'environment-name'], {"env_name": 'environment-name'}),
        ("install", ['install', '--name', 'environment-name'], {"env_name": 'environment-name'}),

        # clone
        (
            "clone",
            ['clone', 'aakashns/jovian-tutorial'],
            {"slug": 'aakashns/jovian-tutorial', "version": None, "include_outputs": True, "overwrite": False}
        ),
        (
            "clone",
            ['clone', 'aakashns/jovian-tutorial', '-v', 3],
            {"slug": 'aakashns/jovian-tutorial', "version": 3, "include_outputs": True, "overwrite": False}
        ),
        (
            "clone",
            ['clone', 'aakashns/jovian-tutorial', '--version', 3],
            {"slug": 'aakashns/jovian-tutorial', "version": 3, "include_outputs": True, "overwrite": False}
        ),
        (
            "clone",
            ['clone', 'aakashns/jovian-tutorial', '-v', 3, '--no-outputs'],
            {"slug": 'aakashns/jovian-tutorial', "version": 3, "include_outputs": False, "overwrite": False}
        ),
        (
            "clone",
            ['clone', 'aakashns/jovian-tutorial', '-v', 3, '--overwrite'],
            {"slug": 'aakashns/jovian-tutorial', "version": 3, "include_outputs": True, "overwrite": True}
        ),

        # pull
        (
            "pull",
            ['pull'],
            {"slug": None, "version": None},
        ),
        (
            "pull",
            ['pull', '-n', 'aakashns/jovian-tutorial'],
            {"slug": 'aakashns/jovian-tutorial', "version": None},
        ),
        (
            "pull",
            ['pull', '-n', 'aakashns/jovian-tutorial', '-v', 3],
            {"slug": 'aakashns/jovian-tutorial', "version": 3},
        ),

        # set-project
        (
            "set_notebook_slug",
            ["set-project", "my_notebook.ipynb", "danb/keras-example"],
            {"filename": "my_notebook.ipynb", "slug": "danb/keras-example"}
        ),

        # commit
        (
            "commit_path",
            ["commit", "my_notebook.ipynb"],
            {"path": "my_notebook.ipynb", "environment": None, "is_cli": True}
        ),
        (
            "commit_path",
            ["commit", "./directory"],
            {"path": "./directory", "environment": None, "is_cli": True}
        )
    ]
)
def test_cli(func, cli_args, called_with_args, runner):
    with mock.patch("jovian.cli.{}".format(func)) as mock_func:
        result = runner.invoke(main, cli_args)
        mock_func.assert_called_with(**called_with_args)
        assert result.exit_code == 0


@mock.patch("jovian.cli.commit_path")
@mock.patch("jovian.cli.log")
@mock.patch("jovian.cli.is_py2", return_value=True)
def test_commit_py2(mock_py2, mock_log, mock_commit, runner):
    result = runner.invoke(main, ["commit", "my_notebook.ipynb"])
    mock_log.assert_called_with(
        "Committing is not supported for Python 2.x. Please install and run Jovian from Python 3.5 and above.",
        warn=True)
    called_with_args = {"path": "my_notebook.ipynb", "environment": None, "is_cli": True}
    mock_commit.assert_called_with(**called_with_args)
    assert result.exit_code == 0



import os
from unittest import TestCase, mock
from unittest.mock import ANY

import pytest

from jovian.tests.resources.shared import MockResponse, fake_creds
from jovian._version import __version__
from jovian.utils.api import (
    _h,
    create_gist_simple,
    get_current_user,
    get_gist,
    get_gist_access,
    post_block,
    post_blocks,
    post_records,
    post_slack_message,
    upload_file,
)
from jovian.utils.credentials import write_creds
from jovian.utils.error import ApiError


def mock_requests_post(url, *args, **kwargs):
    if (
        url == "https://api-staging.jovian.ai/gist/create"
        or url == "https://api-staging.jovian.ai/gist/fake_gist_slug/upload"
    ):
        if kwargs["headers"]["Authorization"] == "Bearer fake_api_key":
            data = {"data": {"message": "Gist created successfully"}}
            return MockResponse(data, status_code=200)
        else:
            data = {
                "errors": [{"code": 404, "message": "Gist not found"}],
                "success": False,
            }
            return MockResponse(data, status_code=404)
    elif url == "https://api-staging.jovian.ai/data/record":
        if kwargs["headers"]["Authorization"] == "Bearer fake_api_key":
            data = {"data": {"message": "Data logged successfully"}}
            return MockResponse(data, status_code=200)
        else:
            data = {
                "errors": [{"code": 500, "message": "Internal Server Error"}],
                "success": False,
            }
            return MockResponse(data, status_code=500)

    elif url == "https://api-staging.jovian.ai/data/fake_gist_slug/commit":
        if kwargs["headers"]["Authorization"] == "Bearer fake_api_key":
            data = {"data": {"message": "Data logged successfully"}}
            return MockResponse(data, status_code=200)
        else:
            data = {
                "errors": [{"code": 404, "message": "Gist not found"}],
                "success": False,
            }
            return MockResponse(data, status_code=404)

    elif url == "https://api-staging.jovian.ai/slack/notify":
        if kwargs["headers"]["Authorization"] == "Bearer fake_api_key":
            data = {
                "data": {
                    "messageId": "60023f08f3b54801bd58cf6b37067ed6",
                    "messageSent": True,
                },
                "success": True,
            }
            return MockResponse(data, 200)
        elif kwargs["headers"]["Authorization"] == "Bearer fake_invalid_api_key":
            data = {
                "errors": [{"code": 401, "message": "The token is invalid"}],
                "success": False,
            }

            return MockResponse(data, 401)

        elif kwargs["headers"]["Authorization"] == "Bearer fake_expired_api_key":
            data = {
                "errors": [{"code": 200, "message": "The token has expired"}],
                "success": False,
            }

            return MockResponse(data, 200)


def mock_requests_get(url, *args, **kwargs):
    if url == "https://api-staging.jovian.ai/user/profile":
        if kwargs["headers"]["Authorization"] == "Bearer fake_api_key":
            data = {"data": {"id": 47, "username": "rohit"}, "success": True}

            return MockResponse(data, status_code=200)
        else:
            data = {
                "errors": [{"code": 401, "message": "Signature verification failed"}],
                "success": False,
            }
            return MockResponse(data, status_code=401)

    elif (
        url == "https://api-staging.jovian.ai/user/rohit/gist/demo-notebook"
        or url
        == "https://api-staging.jovian.ai/user/rohit/gist/demo-notebook?gist_version=3"
        or url == "https://api-staging.jovian.ai/gist/f67108fc906341d8b15209ce88ebc3d2"
    ):

        data = {
            "data": {
                "archived": False,
                "clonesCount": 0,
                "createdAt": 1578718557994,
                "currentUser": {"id": 47, "username": "rohit"},
                "deleted": False,
                "description": None,
            }
        }

        return MockResponse(data, status_code=200)

    # Dictionary containing mock responses
    response_dict = {
        "https://api-staging.jovian.ai/gist/fake_nonexistent_gist": MockResponse(
            {"errors": [{"code": 404, "message": "Gist not found"}], "success": False},
            status_code=404,
        ),
        "https://api-staging.jovian.ai/gist/fake_gist_too_large": MockResponse(
            {"message": "Internal Server Error"}, status_code=500
        ),
        "https://api-staging.jovian.ai/gist/f67108fc906341d8b15209ce88ebc3d2/check-access": MockResponse(
            {
                "data": {
                    "currentUser": {"id": 47, "username": "rohit"},
                    "read": True,
                    "slug": "f67108fc906341d8b15209ce88ebc3d2",
                    "write": True,
                },
                "success": True,
            },
            status_code=200,
        ),
        "https://api-staging.jovian.ai/gist/fake_nonexistent_gist/check-access": MockResponse(
            {"errors": [{"code": 404, "message": "Gist not found"}], "success": False},
            status_code=404,
        ),
    }

    return response_dict[url]


def test_h():
    with fake_creds():
        expected_result = {
            "Authorization": "Bearer fake_api_key",
            "x-jovian-source": "library",
            "x-jovian-library-version": __version__,
            "x-jovian-guest": "b6538d4dfde04fcf993463a828a9cec6",
            "x-jovian-org": "staging",
        }

        assert _h() == expected_result


@mock.patch("requests.get", side_effect=mock_requests_get)
def test_get_current_user(mock_requests_get):
    with fake_creds():
        get_current_user()

        mock_requests_get.assert_called_with(
            "https://api-staging.jovian.ai/user/profile",
            headers={
                "Authorization": "Bearer fake_api_key",
                "x-jovian-source": "library",
                "x-jovian-library-version": __version__,
                "x-jovian-guest": "b6538d4dfde04fcf993463a828a9cec6",
                "x-jovian-org": "staging",
            },
            params=None,
        )


@mock.patch("jovian.utils.request.get_api_key", return_value="fake_invalid_api_key")
@mock.patch("requests.get", side_effect=mock_requests_get)
def test_get_current_user_raises_exception(
    mock_requests_get, mock_request_get_api_key
):
    with fake_creds():
        # setUp
        creds = {
            "WEBAPP_URL": "https://staging.jovian.ml/",
            "GUEST_KEY": "b6538d4dfde04fcf993463a828a9cec6",
            "API_URL": "https://api-staging.jovian.ai",
            "API_KEY": "fake_invalid_api_key",
            "ORG_ID": "staging",
        }
        write_creds(creds)

        with pytest.raises(Exception) as context:
            get_current_user()

        mock_requests_get.assert_called_with(
            "https://api-staging.jovian.ai/user/profile",
            headers={
                "Authorization": "Bearer fake_invalid_api_key",
                "x-jovian-source": "library",
                "x-jovian-library-version": __version__,
                "x-jovian-guest": "b6538d4dfde04fcf993463a828a9cec6",
                "x-jovian-org": "staging",
            },
            params=None,
        )

        assert (
            str(context.value)
            == "Failed to fetch current user profile. (HTTP 401) Signature verification failed"
        )


@mock.patch("jovian.utils.request.get_api_key", return_value="fake_api_key")
@mock.patch("requests.get", side_effect=mock_requests_get)
def test_get_gist(mock_requests_get, mock_request_get_api_key):
    with fake_creds():
        get_gist("rohit/demo-notebook")
        mock_requests_get.assert_called_with(
            "https://api-staging.jovian.ai/user/rohit/gist/demo-notebook",
            headers={
                "Authorization": "Bearer fake_api_key",
                "x-jovian-source": "library",
                "x-jovian-library-version": __version__,
                "x-jovian-guest": "b6538d4dfde04fcf993463a828a9cec6",
                "x-jovian-org": "staging",
            },
            params=None,
        )

        get_gist("rohit/demo-notebook", version=3)
        mock_requests_get.assert_called_with(
            "https://api-staging.jovian.ai/user/rohit/gist/demo-notebook?gist_version=3",
            headers={
                "Authorization": "Bearer fake_api_key",
                "x-jovian-source": "library",
                "x-jovian-library-version": __version__,
                "x-jovian-guest": "b6538d4dfde04fcf993463a828a9cec6",
                "x-jovian-org": "staging",
            },
            params=None,
        )

        get_gist("f67108fc906341d8b15209ce88ebc3d2")
        mock_requests_get.assert_called_with(
            "https://api-staging.jovian.ai/gist/f67108fc906341d8b15209ce88ebc3d2",
            headers={
                "Authorization": "Bearer fake_api_key",
                "x-jovian-source": "library",
                "x-jovian-library-version": __version__,
                "x-jovian-guest": "b6538d4dfde04fcf993463a828a9cec6",
                "x-jovian-org": "staging",
            },
            params=None,
        )


@mock.patch("jovian.utils.request.get_api_key", return_value="fake_api_key")
@mock.patch("requests.get", side_effect=mock_requests_get)
def test_get_gist_not_found(mock_requests_get, mock_get_api_key):
    with fake_creds():
        assert get_gist("fake_nonexistent_gist") == False


@mock.patch("jovian.utils.request.get_api_key", return_value="fake_api_key")
@mock.patch("requests.get", side_effect=mock_requests_get)
def test_get_gist_raises_exception(mock_requests_get, mock_get_api_key):
    with fake_creds():
        with pytest.raises(Exception) as context:
            get_gist("fake_gist_too_large")

        assert (
            str(context.value)
            == 'Failed to retrieve metadata for notebook "fake_gist_too_large":'
            + " (HTTP 500) Internal Server Error"
        )


@mock.patch("jovian.utils.request.get_api_key", return_value="fake_api_key")
@mock.patch("requests.get", side_effect=mock_requests_get)
def test_get_gist_access(mock_requests_get, mock_get_api_key):
    with fake_creds():
        get_gist_access("f67108fc906341d8b15209ce88ebc3d2")
        mock_requests_get.assert_called_with(
            "https://api-staging.jovian.ai/gist/f67108fc906341d8b15209ce88ebc3d2/check-access",
            headers={
                "Authorization": "Bearer fake_api_key",
                "x-jovian-source": "library",
                "x-jovian-library-version": __version__,
                "x-jovian-guest": "b6538d4dfde04fcf993463a828a9cec6",
                "x-jovian-org": "staging",
            },
            params=None,
        )


@mock.patch("jovian.utils.request.get_api_key", return_value="fake_api_key")
@mock.patch("requests.get", side_effect=mock_requests_get)
def test_get_gist_access_raises_exception(
    mock_requests_get, mock_get_api_key
):
    with fake_creds():
        with pytest.raises(Exception) as context:
            get_gist_access("fake_nonexistent_gist")

        mock_requests_get.assert_called_with(
            "https://api-staging.jovian.ai/gist/fake_nonexistent_gist/check-access",
            headers={
                "Authorization": "Bearer fake_api_key",
                "x-jovian-source": "library",
                "x-jovian-library-version": __version__,
                "x-jovian-guest": "b6538d4dfde04fcf993463a828a9cec6",
                "x-jovian-org": "staging",
            },
            params=None,
        )

        assert (
            str(context.value)
            == "Failed to retrieve access permission for notebook"
            + ' "fake_nonexistent_gist" (retry with new_project=True to create a new notebook):'
            + " (HTTP 404) Gist not found"
        )


@mock.patch("requests.post", side_effect=mock_requests_post)
def test_create_gist_simple_no_gist_slug(mock_requests_post):
    with fake_creds() as dir:
        create_gist_simple(
            filename=os.path.join(dir, ".jovian/credentials.json"),
            title="Credentials",
            privacy="private",
            version_title="first version",
        )

        mock_requests_post.assert_called_with(
            "https://api-staging.jovian.ai/gist/create",
            data={
                "visibility": "private",
                "public": False,
                "title": "Credentials",
                "version_title": "first version",
            },
            files={
                "files": (
                    os.path.join(dir, ".jovian/credentials.json"),
                    ANY,
                )
            },
            headers={
                "Authorization": "Bearer fake_api_key",
                "x-jovian-source": "library",
                "x-jovian-library-version": __version__,
                "x-jovian-guest": "b6538d4dfde04fcf993463a828a9cec6",
                "x-jovian-org": "staging",
            },
            json=None,
        )


@mock.patch("requests.post", side_effect=mock_requests_post)
def test_create_gist_simple_with_gist_slug(mock_requests_post):
    with fake_creds() as dir:
        create_gist_simple(
            filename=os.path.join(dir, ".jovian/credentials.json"),
            gist_slug="fake_gist_slug",
            title="Credentials",
            version_title="first version",
        )

        mock_requests_post.assert_called_with(
            "https://api-staging.jovian.ai/gist/fake_gist_slug/upload",
            data={"version_title": "first version"},
            files={
                "files": (
                    os.path.join(dir, ".jovian/credentials.json"),
                    ANY,
                )
            },
            headers={
                "Authorization": "Bearer fake_api_key",
                "x-jovian-source": "library",
                "x-jovian-library-version": __version__,
                "x-jovian-guest": "b6538d4dfde04fcf993463a828a9cec6",
                "x-jovian-org": "staging",
            },
            json=None,
        )


@mock.patch("requests.post", side_effect=mock_requests_post)
def test_create_gist_simple_raises_api_error(mock_requests_post):
    with fake_creds() as dir:
        # setUp
        creds = {
            "WEBAPP_URL": "https://staging.jovian.ml/",
            "GUEST_KEY": "b6538d4dfde04fcf993463a828a9cec6",
            "API_URL": "https://api-staging.jovian.ai",
            "API_KEY": "fake_invalid_api_key",
            "ORG_ID": "staging",
        }
        write_creds(creds)

        with pytest.raises(ApiError) as context:
            create_gist_simple(
                filename=os.path.join(dir, ".jovian/credentials.json"),
                title="Credentials",
                version_title="first version",
            )

        mock_requests_post.assert_called_with(
            "https://api-staging.jovian.ai/gist/create",
            data={
                "visibility": "auto",
                "public": True,
                "title": "Credentials",
                "version_title": "first version",
            },
            files={
                "files": (
                    os.path.join(dir, ".jovian/credentials.json"),
                    ANY,
                )
            },
            headers={
                "Authorization": "Bearer fake_invalid_api_key",
                "x-jovian-source": "library",
                "x-jovian-library-version": __version__,
                "x-jovian-guest": "b6538d4dfde04fcf993463a828a9cec6",
                "x-jovian-org": "staging",
            },
            json=None,
        )

        assert (
            str(context.value)
            == "File upload failed: (HTTP 404) Gist not found"
        )


@mock.patch("requests.post", side_effect=mock_requests_post)
def test_upload_file(mock_requests_post):
    with fake_creds() as dir:
        with open(
            os.path.join(dir, ".jovian/credentials.json"), "rb"
        ) as f:
            upload_file(
                gist_slug="fake_gist_slug",
                file=("credentials.json", f),
                folder=".jovian",
                artifact=True,
                version_title="fake_version_title",
            )

            mock_requests_post.assert_called_with(
                "https://api-staging.jovian.ai/gist/fake_gist_slug/upload",
                data={
                    "artifact": "true",
                    "folder": ".jovian",
                    "version_title": "fake_version_title",
                },
                files={"files": ("credentials.json", ANY)},
                headers={
                    "Authorization": "Bearer fake_api_key",
                    "x-jovian-source": "library",
                    "x-jovian-library-version": __version__,
                    "x-jovian-guest": "b6538d4dfde04fcf993463a828a9cec6",
                    "x-jovian-org": "staging",
                },
                json=None,
            )


@mock.patch("jovian.utils.request.get_api_key", return_value="fake_invalid_api_key")
@mock.patch("requests.post", side_effect=mock_requests_post)
def test_upload_file_raises_api_error(mock_requests_post, mock_get_api_key):
    with fake_creds() as dir:
        # setUp
        creds = {
            "WEBAPP_URL": "https://staging.jovian.ml/",
            "GUEST_KEY": "b6538d4dfde04fcf993463a828a9cec6",
            "API_URL": "https://api-staging.jovian.ai",
            "API_KEY": "fake_invalid_api_key",
            "ORG_ID": "staging",
        }
        write_creds(creds)

        with pytest.raises(ApiError) as context:
            with open(
                os.path.join(dir, ".jovian/credentials.json"), "rb"
            ) as f:
                upload_file(
                    gist_slug="fake_gist_slug",
                    file=("credentials.json", f),
                    folder=".jovian",
                    artifact=True,
                    version_title="fake_version_title",
                )

        mock_requests_post.assert_called_with(
            "https://api-staging.jovian.ai/gist/fake_gist_slug/upload",
            data={
                "artifact": "true",
                "folder": ".jovian",
                "version_title": "fake_version_title",
            },
            files={"files": ("credentials.json", ANY)},
            headers={
                "Authorization": "Bearer fake_invalid_api_key",
                "x-jovian-source": "library",
                "x-jovian-library-version": __version__,
                "x-jovian-guest": "b6538d4dfde04fcf993463a828a9cec6",
                "x-jovian-org": "staging",
            },
            json=None,
        )

        assert (
            str(context.value)
            == "File upload failed: (HTTP 404) Gist not found"
        )


@mock.patch("requests.post", side_effect=mock_requests_post)
def test_post_blocks(mock_requests_post):

    with fake_creds():
        blocks = [{"data": {"key": "value"}, "record_type": "metrics"}]
        post_blocks(blocks)

        mock_requests_post.assert_called_with(
            "https://api-staging.jovian.ai/data/record",
            data=None,
            headers={
                "Authorization": "Bearer fake_api_key",
                "x-jovian-source": "library",
                "x-jovian-library-version": __version__,
                "x-jovian-guest": "b6538d4dfde04fcf993463a828a9cec6",
                "x-jovian-org": "staging",
            },
            json=[{"data": {"key": "value"}, "record_type": "metrics"}],
        )


@mock.patch("jovian.utils.request.get_api_key", return_value="fake_invalid_api_key")
@mock.patch("requests.post", side_effect=mock_requests_post)
def test_post_blocks_raises_api_error(mock_requests_post, mock_get_api_key):
    with fake_creds():
        # setUp
        creds = {
            "WEBAPP_URL": "https://staging.jovian.ml/",
            "GUEST_KEY": "b6538d4dfde04fcf993463a828a9cec6",
            "API_URL": "https://api-staging.jovian.ai",
            "API_KEY": "fake_invalid_api_key",
            "ORG_ID": "staging",
        }
        write_creds(creds)

        blocks = [{"data": {"key": "value"}, "record_type": "metrics"}]

        with pytest.raises(ApiError) as context:
            post_blocks(blocks)

        mock_requests_post.assert_called_with(
            "https://api-staging.jovian.ai/data/record",
            data=None,
            headers={
                "Authorization": "Bearer fake_invalid_api_key",
                "x-jovian-source": "library",
                "x-jovian-library-version": __version__,
                "x-jovian-guest": "b6538d4dfde04fcf993463a828a9cec6",
                "x-jovian-org": "staging",
            },
            json=[{"data": {"key": "value"}, "record_type": "metrics"}],
        )

        assert (
            str(context.value)
            == "Data logging failed: (HTTP 500) Internal Server Error"
        )


@mock.patch("jovian.utils.api.timestamp_ms", return_value=1582550133094)
@mock.patch("requests.post", side_effect=mock_requests_post)
def test_post_block(mock_requests_post, mock_timestamp):
    with fake_creds():
        post_block("metrics", {"key": "value"})

        mock_requests_post.assert_called_with(
            "https://api-staging.jovian.ai/data/record",
            data=None,
            headers={
                "Authorization": "Bearer fake_api_key",
                "x-jovian-source": "library",
                "x-jovian-library-version": __version__,
                "x-jovian-guest": "b6538d4dfde04fcf993463a828a9cec6",
                "x-jovian-org": "staging",
            },
            json=[
                {
                    "localTimestamp": 1582550133094,
                    "data": "metrics",
                    "recordType": {"key": "value"},
                }
            ],
        )


@mock.patch("jovian.utils.api.timestamp_ms", return_value=1582550133094)
@mock.patch("jovian.utils.request.get_api_key", return_value="fake_invalid_api_key")
@mock.patch("requests.post", side_effect=mock_requests_post)
def test_post_block_raises_api_error(
    mock_requests_post, mock_get_api_key, mock_timestamp
):
    with fake_creds():
        # setUp
        creds = {
            "WEBAPP_URL": "https://staging.jovian.ml/",
            "GUEST_KEY": "b6538d4dfde04fcf993463a828a9cec6",
            "API_URL": "https://api-staging.jovian.ai",
            "API_KEY": "fake_invalid_api_key",
            "ORG_ID": "staging",
        }
        write_creds(creds)

        with pytest.raises(ApiError) as context:
            post_block("metrics", {"key": "value"})

        mock_requests_post.assert_called_with(
            "https://api-staging.jovian.ai/data/record",
            data=None,
            headers={
                "Authorization": "Bearer fake_invalid_api_key",
                "x-jovian-source": "library",
                "x-jovian-library-version": __version__,
                "x-jovian-guest": "b6538d4dfde04fcf993463a828a9cec6",
                "x-jovian-org": "staging",
            },
            json=[
                {
                    "localTimestamp": 1582550133094,
                    "data": "metrics",
                    "recordType": {"key": "value"},
                }
            ],
        )

        assert (
            str(context.value)
            == "Data logging failed: (HTTP 500) Internal Server Error"
        )


@mock.patch("requests.post", side_effect=mock_requests_post)
def test_post_records(mock_requests_post):
    with fake_creds():
        post_records("fake_gist_slug", {"key": "value"})
        mock_requests_post.assert_called_with(
            "https://api-staging.jovian.ai/data/fake_gist_slug/commit",
            data=None,
            headers={
                "Authorization": "Bearer fake_api_key",
                "x-jovian-source": "library",
                "x-jovian-library-version": __version__,
                "x-jovian-guest": "b6538d4dfde04fcf993463a828a9cec6",
                "x-jovian-org": "staging",
            },
            json={"key": "value"},
        )


@mock.patch("jovian.utils.request.get_api_key", return_value="fake_invalid_api_key")
@mock.patch("requests.post", side_effect=mock_requests_post)
def test_post_records_raises_api_error(mock_requests_post, mock_get_api_key):
    with fake_creds():
        # setUp
        creds = {
            "WEBAPP_URL": "https://staging.jovian.ml/",
            "GUEST_KEY": "b6538d4dfde04fcf993463a828a9cec6",
            "API_URL": "https://api-staging.jovian.ai",
            "API_KEY": "fake_invalid_api_key",
            "ORG_ID": "staging",
        }
        write_creds(creds)

        with pytest.raises(ApiError) as context:
            post_records("fake_gist_slug", {"key": "value"})

        mock_requests_post.assert_called_with(
            "https://api-staging.jovian.ai/data/fake_gist_slug/commit",
            data=None,
            headers={
                "Authorization": "Bearer fake_invalid_api_key",
                "x-jovian-source": "library",
                "x-jovian-library-version": __version__,
                "x-jovian-guest": "b6538d4dfde04fcf993463a828a9cec6",
                "x-jovian-org": "staging",
            },
            json={"key": "value"},
        )

        assert (
            str(context.value)
            == "Data logging failed: (HTTP 404) Gist not found"
        )


@mock.patch("requests.post", side_effect=mock_requests_post)
def test_post_slack_message(mock_requests_post):
    with fake_creds():
        # setUp
        creds = {
            "WEBAPP_URL": "https://staging.jovian.ml/",
            "GUEST_KEY": "b6538d4dfde04fcf993463a828a9cec6",
            "API_URL": "https://api-staging.jovian.ai",
            "API_KEY": "fake_api_key",
            "ORG_ID": "staging",
        }
        write_creds(creds)

        post_slack_message({"key": "value"})

        mock_requests_post.assert_called_with(
            "https://api-staging.jovian.ai/slack/notify",
            data=None,
            headers={
                "Authorization": "Bearer fake_api_key",
                "x-jovian-source": "library",
                "x-jovian-library-version": __version__,
                "x-jovian-guest": "b6538d4dfde04fcf993463a828a9cec6",
                "x-jovian-org": "staging",
            },
            json={"key": "value"},
        )


@mock.patch("jovian.utils.request.get_api_key", return_value="fake_invalid_api_key")
@mock.patch("requests.post", side_effect=mock_requests_post)
def test_post_slack_message_raises_api_error(
    mock_requests_post, mock_request_get_api_key
):
    with fake_creds():
        # setUp
        creds = {
            "WEBAPP_URL": "https://staging.jovian.ml/",
            "GUEST_KEY": "b6538d4dfde04fcf993463a828a9cec6",
            "API_URL": "https://api-staging.jovian.ai",
            "API_KEY": "fake_invalid_api_key",
            "ORG_ID": "staging",
        }
        write_creds(creds)

        data = {"key": "value"}

        with pytest.raises(ApiError):
            post_slack_message(data)

        mock_requests_post.assert_called_with(
            "https://api-staging.jovian.ai/slack/notify",
            data=None,
            headers={
                "Authorization": "Bearer fake_invalid_api_key",
                "x-jovian-source": "library",
                "x-jovian-library-version": __version__,
                "x-jovian-guest": "b6538d4dfde04fcf993463a828a9cec6",
                "x-jovian-org": "staging",
            },
            json={"key": "value"},
        )


@mock.patch("jovian.utils.request.get_api_key", return_value="fake_invalid_api_key")
@mock.patch("requests.post", side_effect=mock_requests_post)
def test_post_slack_message_safe(
    mock_requests_post, mock_request_get_api_key
):
    with fake_creds():
        # setUp
        creds = {
            "WEBAPP_URL": "https://staging.jovian.ml/",
            "GUEST_KEY": "b6538d4dfde04fcf993463a828a9cec6",
            "API_URL": "https://api-staging.jovian.ai",
            "API_KEY": "fake_invalid_api_key",
            "ORG_ID": "staging",
        }
        write_creds(creds)

        data = {"key": "value"}

        assert post_slack_message(data, safe=True) == {
            "data": {"messageSent": False}
        }

        mock_requests_post.assert_called_with(
            "https://api-staging.jovian.ai/slack/notify",
            data=None,
            headers={
                "Authorization": "Bearer fake_invalid_api_key",
                "x-jovian-source": "library",
                "x-jovian-library-version": __version__,
                "x-jovian-guest": "b6538d4dfde04fcf993463a828a9cec6",
                "x-jovian-org": "staging",
            },
            json={"key": "value"},
        )



import sys
from unittest import TestCase, mock

import pytest

from jovian._version import __version__
from jovian.utils.constants import LINUX, MACOS, WINDOWS
from jovian.utils.misc import (get_file_extension, get_flavor, get_platform, is_flavor_pro, is_uuid, timestamp_ms,
                               urljoin, version)


@pytest.mark.parametrize(
    "uuid, expected_result",
    [
        ("374ab6086ca449769b3e44b6e35f9126", True),
        ("374ab608-6ca4-4976-9b3e-44b6e35f9126", True),
        ("this-is-not-a-uuid", False),
    ]
)
def test_is_uuid(uuid, expected_result):
    assert is_uuid(uuid) == expected_result


@pytest.mark.parametrize(
    "filename, expected_result",
    [
        ("file.ipynb", ".ipynb"),
        ("file.txt", ".txt"),
        ("file.py", ".py"),
        ("file.csv", ".csv"),
        ("file", ""),
    ]
)
def test_get_file_extension(filename, expected_result):
    assert get_file_extension(filename) == expected_result


@pytest.mark.parametrize(
    "ret_val, expected_result",
    [
        ("Windows", WINDOWS),
        ("Linux", LINUX),
        ("Darwin", MACOS),
    ]
)
def test_get_platform(ret_val, expected_result):
    with mock.patch("platform.system", mock.Mock(return_value=ret_val)):
        assert get_platform() == expected_result


@pytest.mark.parametrize(
    "urls, expected_result",
    [
        (["https://jovian.ml"], "https://jovian.ml"),
        (["https://jovian.ml/"], "https://jovian.ml/"),
        (["///user/siddhant"], "user/siddhant"),
        (["https://jovian.ml", "user/siddhant"], "https://jovian.ml/user/siddhant"),
        (["https://jovian.ml", "/user/siddhant"], "https://jovian.ml/user/siddhant"),
        (["https://jovian.ml/", "user/siddhant"], "https://jovian.ml/user/siddhant"),
        (["https://jovian.ml/", "/user/siddhant"], "https://jovian.ml/user/siddhant"),
        (["https://jovian.ml", "user/siddhant/"], "https://jovian.ml/user/siddhant/"),
        (["https://jovian.ml", "/user/siddhant/"], "https://jovian.ml/user/siddhant/"),
        (["https://jovian.ml/", "user/siddhant/"], "https://jovian.ml/user/siddhant/"),
        (["https://jovian.ml/", "/user/siddhant/"], "https://jovian.ml/user/siddhant/"),
        (["https://jovian.ml/", "///user/siddhant/////"], "https://jovian.ml/user/siddhant/"),
        (["https://jovian.ml/", "user/siddhant/", "/gists/starred"], "https://jovian.ml/user/siddhant/gists/starred"),
    ]
)
def test_urljoin(urls, expected_result):
    assert urljoin(*urls) == expected_result


def test_urljoin_no_args():
    with pytest.raises(TypeError):
        urljoin()


@mock.patch('time.time', mock.Mock(return_value=1577112328.529031))
def test_timestamp_ms():
    assert timestamp_ms() == 1577112328529


def test_get_flavor():
    assert get_flavor() == "jovian"


def test_get_flavor_import_error():
    import builtins
    _original_import = builtins.__import__

    def error_raiser(*args, **kwargs):
        raise ImportError('fake import error')

    builtins.__import__ = error_raiser
    flavor = get_flavor()
    builtins.__import__ = _original_import

    assert flavor == "jovian"


def test_is_flavor_pro_false():
    import jovian._flavor
    jovian._flavor.__flavor__ = 'jovian'
    assert not is_flavor_pro()


def test_is_flavor_pro_true():
    import jovian._flavor
    jovian._flavor.__flavor__ = 'jovian-pro'
    assert is_flavor_pro()


def test_version():
    assert version() == __version__



import os
import shutil
import textwrap
from unittest import TestCase, mock
from contextlib import contextmanager
from subprocess import check_call
from jovian.tests.resources.shared import temp_directory, mock_git_repo
from jovian.utils.git import (commit, get_branch, get_current_commit, get_relative_path, get_remote,
                              get_repository_root, git_commit_push, git_push, is_git)


def test_is_git():
    with mock_git_repo():
        assert is_git()


def test_is_not_git():
    with temp_directory():
        assert not is_git()


def test_get_branch():
    with mock_git_repo():
        assert get_branch() == "master"

        os.system('git checkout -b sample_branch')
        assert get_branch() == "sample_branch"


def test_get_remote():
    with mock_git_repo():
        expected_result = 'https://github.com/JovianML/mock_repo'
        assert get_remote() == expected_result


def test_get_repository_root():
    with mock_git_repo():
        assert get_repository_root().endswith('mock_git_repo')


def test_get_current_commit():
    with mock_git_repo():
        expected_result = os.popen('git rev-parse HEAD').read().strip()

        assert get_current_commit() == expected_result


def test_commit():
    with mock_git_repo():
        os.system('touch file')
        message = 'sample commit'
        assert commit(message) == 0


@mock.patch("os.system")
def test_git_push_master(mock_system):
    with mock_git_repo():
        git_push()
        mock_system.assert_called_with("git push origin master")


@mock.patch("os.system")
def test_git_push_other_branch(mock_system):
    with mock_git_repo():
        check_call("git checkout -b sample_branch".split())

        git_push()
        mock_system.assert_called_with("git push origin sample_branch")


@mock.patch("jovian.utils.git.git_push")
def test_git_commit_push(mock_git_push):
    with mock_git_repo():
        message = 'sample commit'
        expected_result = {
            'remote': get_remote(),
            'branch': get_branch(),
            'commit': get_current_commit()
        }

        assert git_commit_push(message) == expected_result
        mock_git_push.assert_called_with()


def test_get_relative_path():
    with mock_git_repo():
        os.makedirs("nested/folder/deep")
        os.chdir("nested/folder/deep")

        assert get_relative_path() == "nested/folder/deep"



from unittest import TestCase, mock

import pytest

import jovian.utils.records
from jovian.tests.resources.shared import fake_records
from jovian.utils.records import get_records, log_dataset, log_git, log_hyperparams, log_metrics, log_record, reset

RECORDS = [('fake_slug_metrics_1', 'metrics', {}),
           ('fake_slug_metrics_2', 'metrics', {}),
           ('fake_slug_hyperparams_1', 'hyperparams', {}),
           ('fake_slug_hyperparams_2', 'hyperparams', {})]


def test_get_records_without_type():
    with fake_records():
        expected_result = ['fake_slug_metrics_1',
                           'fake_slug_metrics_2',
                           'fake_slug_hyperparams_1',
                           'fake_slug_hyperparams_2']

        assert get_records(slug_only=True) == expected_result


def test_get_records_with_type():
    with fake_records():
        expected_result = [('fake_slug_metrics_1', 'metrics', {}),
                           ('fake_slug_metrics_2', 'metrics', {}),
                           ('fake_slug_hyperparams_1', 'hyperparams', {}),
                           ('fake_slug_hyperparams_2', 'hyperparams', {})]
        assert get_records() == expected_result


@pytest.mark.parametrize(
    "metric_type, expected_result",
    [
        (
            ['metrics'],
            [('fake_slug_hyperparams_1', 'hyperparams', {}),
             ('fake_slug_hyperparams_2', 'hyperparams', {})]
        ),
        (
            ['hyperparams'],
            [('fake_slug_metrics_1', 'metrics', {}),
             ('fake_slug_metrics_2', 'metrics', {})]
        ),
        (
            ['metrics', 'hyperparams'],
            []
        ),
    ]
)
def test_reset(metric_type, expected_result):
    with fake_records():
        reset(*metric_type)
        assert jovian.utils.records._data_blocks == expected_result


def test_reset_all():
    with fake_records():
        reset()
        assert jovian.utils.records._data_blocks == []


def mock_api_post_block(*args, **kwargs):
    data = {
        'count': 1,
        'tracking': {
            'createdAt': 1577270059593,
            'updatedAt': None, 'gistSlug': None,
            'trackingSlug': 'fake_slug_3',
            'gistVersion': None
        }
    }

    return data


@mock.patch("jovian.utils.records.api.post_block", side_effect=mock_api_post_block)
def test_log_record_no_data(mock_api_post_block):
    with fake_records():
        data = {}
        expected_result = RECORDS

        log_record('fake_record_type', data)
        assert jovian.utils.records._data_blocks == expected_result


@pytest.mark.parametrize(
    "data, func, metric_type",
    [
        (
            {'arch_name': 'cnn_1', 'lr': .001},
            log_hyperparams,
            'hyperparams',
        ),
        (
            {'acc': 0.89, 'val_acc': 0.86},
            log_metrics,
            'metrics',
        ),
        (
            {'col1': [1, 2, 3], 'col2': [4, 5, 6]},
            log_dataset,
            'dataset',
        ),
        (
            {
                'branch': 'sample_branch',
                'commit': 'sample_commit_hash',
                'remote': 'https://github.com/rohitsanj/fakerepo'
            },
            log_git,
            'git'
        ),
    ]
)
@mock.patch("jovian.utils.records.api.post_block", side_effect=mock_api_post_block)
def test_log_records(mock_api_post_block, data, func, metric_type, capsys):
    with fake_records():
        func(data)
        assert jovian.utils.records._data_blocks == RECORDS + [('fake_slug_3', metric_type, data)]

        captured = capsys.readouterr()
        expected_result = '[jovian] {} logged.'.format(metric_type.capitalize())
        assert expected_result == captured.out.strip()


@pytest.mark.parametrize(
    "data, data_args, expected_result",
    [
        (
            {"key1": "value1"},
            {"key2": "value2"},
            {"key1": "value1", "key2": "value2"},
        ),
        (
            ["foo", "bar"],
            {"spam": "eggs"},
            ['foo', 'bar', {'spam': 'eggs'}],
        ),
    ]
)
@mock.patch("jovian.utils.records.api.post_block", side_effect=mock_api_post_block)
def test_log_record_with_data_args(mock_api_post_block, data, data_args, expected_result):
    with fake_records():
        log_record('fake_record_type', data, **data_args)
        assert jovian.utils.records._data_blocks == RECORDS + [('fake_slug_3', 'fake_record_type', expected_result)]



import os
from textwrap import dedent
from unittest import TestCase, mock
from unittest.mock import ANY, call
from contextlib import contextmanager

import pytest


from jovian.utils.install import run_command, install, activate
from jovian.tests.resources.shared import temp_directory, fake_envfile


@pytest.mark.parametrize(
    "side_effect, calls",
    [
        (
            [(None, b'')],
            [
                call('conda env update --file environment.yml --name environment-name', shell=True, stderr=ANY),
                call().communicate()
            ]
        ),
        (
            [(None, b"""ResolvePackageNotFound: \n- mixpanel=1.11.0"""), (None, b'')],
            [
                call('conda env update --file environment.yml --name environment-name', shell=True, stderr=ANY),
                call().communicate(),
                call('conda env update --file environment.yml --name environment-name', shell=True, stderr=-1),
                call().communicate(),
            ]
        ),
        (
            [(None, b"""ResolvePackageNotFound: \n- mixpanel=1.11.0"""),
             (None, b"""ResolvePackageNotFound: \n- mixpanel=1.11.0"""),
             (None, b"""ResolvePackageNotFound: \n- mixpanel=1.11.0""")],
            [
                call('conda env update --file environment.yml --name environment-name', shell=True, stderr=ANY),
                call().communicate(),
                call('conda env update --file environment.yml --name environment-name', shell=True, stderr=ANY),
                call().communicate(),
                call('conda env update --file environment.yml --name environment-name', shell=True, stderr=ANY),
                call().communicate()
            ]
        ),
        (
            [(None, b"""Pip failed""")],
            [
                call('conda env update --file environment.yml --name environment-name', shell=True, stderr=ANY),
                call().communicate()
            ]
        ),
    ]
)
@mock.patch("subprocess.Popen")
def test_run_command(mock_popen, side_effect, calls):
    with fake_envfile():
        mock_popen().communicate.side_effect = side_effect
        run_command('conda env update --file environment.yml --name environment-name',
                    'environment.yml', ['mixpanel=1.11.0', 'sigmasix=1.91.0'])

        mock_popen.assert_has_calls(calls)


@mock.patch("jovian.utils.install.get_conda_bin", return_value="conda")
@mock.patch("jovian.utils.install.request_env_name", return_value="test-env")
@mock.patch("jovian.utils.install.run_command")
def test_install(mock_run_command, mock_request_env_name, mock_get_conda_bin):
    with fake_envfile():
        mock_run_command.return_value = True

        install('environment.yml')

        mock_run_command.assert_called_with(
            command='conda env update --file "environment.yml" --name "test-env"',
            env_fname='environment.yml',
            packages=['mixpanel=1.11.0', 'sigmasix=1.91.0', 'sqlite', 'six==1.11.0', 'sqlite==2.0.0'],
            run=1)


@mock.patch("jovian.utils.install.get_conda_bin", return_value="conda")
@mock.patch("jovian.utils.install.identify_env_file", return_value=None)
def test_install_env_fname_none(mock_identify_env_file, mock_get_conda_bin, capsys):
    install()

    expected_result = "[jovian] Error: Failed to detect a conda environment YML file. Skipping.."
    captured = capsys.readouterr()
    assert captured.err.strip() == expected_result


@mock.patch("jovian.utils.install.get_conda_bin", return_value="conda")
@mock.patch("jovian.utils.install.request_env_name", return_value=None)
def test_install_env_name_none(mock_request_env_name, mock_get_conda_bin, capsys):
    with fake_envfile():
        install('environment.yml')

        expected_result = dedent("""
            [jovian] Detected conda environment file: environment.yml

            [jovian] Environment name not provided/detected. Skipping..
            """).strip()
        captured = capsys.readouterr()
        assert captured.out.strip() == expected_result


@mock.patch("jovian.utils.install.get_conda_bin", return_value="conda")
@mock.patch("jovian.utils.install.request_env_name", return_value="test-env")
@mock.patch("jovian.utils.install.run_command")
def test_install_unsuccessful(mock_run_command, mock_request_env_name, mock_get_conda_bin, capsys):
    with fake_envfile():
        mock_run_command.return_value = False

        install('environment.yml')

        expected_result = dedent("""
                            [jovian] Detected conda environment file: environment.yml

                            [jovian] Some pip packages failed to install.
                            [jovian] 
                            #
                            # To activate this environment, use
                            #
                            #     $ conda activate test-env
                            #
                            # To deactivate an active environment, use
                            #
                            #     $ conda deactivate
                        """).strip()

        captured = capsys.readouterr()
        assert captured.out.strip() == expected_result


@mock.patch("jovian.utils.install.get_conda_bin", return_value="conda")
def test_activate(mock_get_conda_bin, capsys):
    with fake_envfile():
        activate('environment.yml')

        expected_result = dedent("""
            [jovian] Detected conda environment file: environment.yml

            [jovian] Copy and execute the following command (try "source activate" if "conda activate doesn't work" )
                conda activate test-env
        """).strip()

        captured = capsys.readouterr()
        assert expected_result.strip() in captured.out.strip()


def error_raiser(*args, **kwargs):
    raise Exception('fake error')


@mock.patch("jovian.utils.install.get_conda_bin", side_effect=error_raiser)
def test_activate_conda_not_found(mock_get_conda_bin, capsys):
    with fake_envfile():
        assert activate('environment.yml') == False

        expected_result = """[jovian] Error: Anaconda binary not found. Please make sure the "conda" command is in your system PATH or the environment variable $CONDA_EXE points to the anaconda binary"""

        captured = capsys.readouterr()
        assert expected_result.strip() in captured.err.strip()


@mock.patch("jovian.utils.install.identify_env_file", return_value=None)
@mock.patch("jovian.utils.install.get_conda_bin", return_value="conda")
def test_activate_env_fname_none(mock_get_conda_bin, mock_identify_env_file, capsys):
    with fake_envfile():
        assert activate('environment.yml') == False

        expected_result = """[jovian] Error: Failed to detect a conda environment YML file. Skipping.."""

        captured = capsys.readouterr()
        assert expected_result.strip() in captured.err.strip()


@mock.patch("jovian.utils.install.extract_env_name", return_value=None)
@mock.patch("jovian.utils.install.get_conda_bin", return_value="conda")
def test_activate_env_name_none(mock_get_conda_bin, mock_extract_env_name, capsys):
    with fake_envfile():
        activate('environment.yml')

        expected_result = """[jovian] Environment name not provided/detected. Skipping.."""

        captured = capsys.readouterr()
        assert expected_result.strip() in captured.out.strip()



from jovian.tests.resources.shared import fake_creds
from jovian.utils.shared import _u, _v


def test_v():
    assert _v(3) == "?gist_version=3"
    assert _v(None) == ""


def test_u():
    with fake_creds():
        path = 'user/profile'

        assert _u(path) == 'https://api-staging.jovian.ai/user/profile'



import json
from unittest import TestCase, mock

import pytest

from jovian.tests.resources.shared import MockResponse
from jovian.utils.request import _msg, get, post, pretty, retry


def test_msg_errors():
    mock_res = mock.Mock()
    mock_res.json.return_value = {
        'errors': [
            {
                'message': 'this is a fake requests error',
                'code': 400
            }
        ]
    }

    expected_result = 'this is a fake requests error'
    assert _msg(mock_res) == expected_result


def test_msg_message():
    mock_res = mock.Mock()
    mock_res.json.return_value = {
        'message': 'this is a fake response message'
    }

    expected_result = 'this is a fake response message'
    assert _msg(mock_res) == expected_result


def test_msg_msg():
    mock_res = mock.Mock()
    mock_res.json.return_value = {
        'msg': 'this is a fake response message'
    }

    expected_result = 'this is a fake response message'
    assert _msg(mock_res) == expected_result


def test_msg_exception_text():
    mock_res = mock.Mock(return_value=ValueError('fake value error'))
    mock_res.text = 'this is a fake response text'

    expected_result = 'this is a fake response text'
    assert _msg(mock_res) == expected_result


def test_msg_exception_no_text():
    mock_res = mock.Mock()
    mock_res.json.return_value = ValueError('fake value error')
    mock_res.text = ''
    expected_result = 'Something went wrong'
    assert _msg(mock_res) == expected_result


def test_pretty():
    mock_res = mock.Mock()
    mock_res.status_code = 200
    mock_res.json.return_value = {
        'msg': 'this is a fake response message'
    }

    expected_result = '(HTTP 200) this is a fake response message'
    assert pretty(mock_res) == expected_result


def mocked_requests(url, *args, **kwargs):
    if url == 'http://someurl.com/test.json':
        return MockResponse({'key': 'value'}, 200)
    elif url == 'http://someurl.com/noauth':
        return MockResponse({'message': 'invalid creds'}, 401)
    elif url == 'http://someurl.com/create':
        return MockResponse({'message': 'created successfully'}, 201)
    return MockResponse(None, 404)


@pytest.mark.parametrize(
    "func, url, req, ret_val_api_key, call_count",
    [
        (
            get,
            "http://someurl.com/test.json",
            "requests.get",
            None,
            1
        ),
        (
            get,
            "http://someurl.com/noauth",
            "requests.get",
            "fake_api_key",
            2
        ),
        (
            post,
            "http://someurl.com/noauth",
            "requests.post",
            "fake_api_key",
            2
        ),
        (
            post,
            "http://someurl.com/create",
            "requests.post",
            None,
            1
        ),
    ]
)
def test_request(func, url, req, ret_val_api_key, call_count):
    with mock.patch(req, side_effect=mocked_requests) as mock_req, \
            mock.patch('jovian.utils.request.get_api_key', return_value=ret_val_api_key):

        func(url, headers={'key': 'value'})
        assert mock_req.call_count == call_count



import os
import shutil
from contextlib import contextmanager
from unittest import mock
from unittest.mock import call
from textwrap import dedent

import pytest

from jovian.tests.resources.shared import MockResponse, fake_creds, temp_directory
from jovian.utils.clone import _h, clone, get_gist, post_clone_msg, pull
from jovian import __version__

HEADERS = {"Authorization": "Bearer fake_api_key",
           "x-jovian-source": "library",
           "x-jovian-library-version": __version__,
           "x-jovian-command": "clone",
           "x-jovian-guest": "b6538d4dfde04fcf993463a828a9cec6",
           "x-jovian-org": "staging"}

FAKE_GIST = {
    "title": "metrics-example",
    "files": [
        {
            "filename": "metrics-example.ipynb",
            "artifact": False,
            "rawUrl": "https://storage.com/slug1"
        },
        {
            "filename": "environment.yml",
            "artifact": False,
            "rawUrl": "https://storage.com/slug2"
        },
    ],
}


@pytest.mark.parametrize(
    "command, fresh",
    [
        ("clone", True),
        ("pull", False),
    ]
)
def test_h(command, fresh):
    with fake_creds():
        expected_result = {"Authorization": "Bearer fake_api_key",
                           "x-jovian-source": "library",
                           "x-jovian-library-version": __version__,
                           "x-jovian-command": command,
                           "x-jovian-guest": "b6538d4dfde04fcf993463a828a9cec6",
                           "x-jovian-org": "staging"}

        assert _h(fresh=fresh) == expected_result


@pytest.mark.parametrize(
    "gist, called_with_url",
    [
        (
            'aakashns/jovian-tutorial',
            'https://api-staging.jovian.ai/user/aakashns/gist/jovian-tutorial?gist_version=3',
        ),
        (
            'fake_gist_slug',
            'https://api-staging.jovian.ai/gist/fake_gist_slug?gist_version=3',
        ),
    ]
)
@mock.patch("jovian.utils.clone.get", return_value=MockResponse({'data': {'key': 'value'}}, 200))
def test_get_gist(mock_get, gist, called_with_url):
    with fake_creds():
        get_gist(gist, 3, fresh=True)

        mock_get.assert_called_with(called_with_url, headers=HEADERS)


@mock.patch("jovian.utils.clone.get", return_value=MockResponse({'data': {'key': 'value'}}, 404))
def test_get_gist_raises_exception(mock_get):
    with fake_creds():
        with pytest.raises(Exception):
            get_gist('does-not-exist', 3, fresh=True)


def test_post_clone_message(capsys):
    post_clone_msg('jovian-tutorial')

    expected_result = dedent("""
            [jovian] Cloned successfully to 'jovian-tutorial'

            Next steps:
              $ cd jovian-tutorial
              $ jovian install
              $ conda activate <env_name>
              $ jupyter notebook


            Replace <env_name> with the name of your environment (without the '<' & '>')
            Jovian uses Anaconda ( https://conda.io/ ) under the hood,
            so please make sure you have it installed and added to path.
            * If you face issues with `jovian install`, try `conda env update`.
            * If you face issues with `conda activate`, try `source activate <env_name>`
              or `activate <env_name>` to activate the virtual environment.""").strip()

    captured = capsys.readouterr()
    assert captured.out.strip() == expected_result.strip()


@mock.patch("jovian.utils.clone.get")
@mock.patch("jovian.utils.clone.get_gist")
def test_clone(mock_get_gist, mock_requests_get):
    with temp_directory() as dir:
        mock_get_gist.return_value = FAKE_GIST

        get1, get2 = mock.Mock(), mock.Mock()
        get1.content, get2.content = b"notebook content", b"environment content"

        mock_requests_get.side_effect = [get1, get2]

        clone('aakashns/metrics-example', version='3')

        os.chdir(dir)

        mock_get_gist.assert_called_with('aakashns/metrics-example', '3', True)
        mock_requests_get.assert_has_calls([
            call("https://storage.com/slug1"),
            call("https://storage.com/slug2")
        ])

        # Check that folder was created
        assert os.listdir() == ["metrics-example"]

        # Check that files were downloaded
        assert set(os.listdir("metrics-example")) == {".jovianrc", "environment.yml", "metrics-example.ipynb"}


@mock.patch("jovian.utils.clone.get")
@mock.patch("jovian.utils.clone.get_gist")
def test_clone_multiple(mock_get_gist, mock_requests_get):
    with temp_directory() as dir:
        mock_get_gist.return_value = FAKE_GIST

        get1, get2 = mock.Mock(), mock.Mock()
        get1.content, get2.content = b"notebook content", b"environment content"

        mock_requests_get.side_effect = [get1, get2] * 3

        # first call
        clone('aakashns/metrics-example', version='3')
        os.chdir(dir)

        # second call
        clone('aakashns/metrics-example', version='3')
        os.chdir(dir)

        # third call
        clone('aakashns/metrics-example', version='3')
        os.chdir(dir)

        mock_get_gist.assert_called_with('aakashns/metrics-example', '3', True)
        mock_requests_get.assert_has_calls([
            call("https://storage.com/slug1"),
            call("https://storage.com/slug2")
        ] * 3)

        # Check that multiple folders were created
        assert set(os.listdir()) == {"metrics-example", "metrics-example-1", "metrics-example-2"}


@mock.patch("jovian.utils.clone.get")
@mock.patch("jovian.utils.clone.get_gist")
def test_clone_fresh_false(mock_get_gist, mock_requests_get):
    with temp_directory() as dir:
        mock_get_gist.return_value = FAKE_GIST

        get1, get2 = mock.Mock(), mock.Mock()
        get1.content, get2.content = b"notebook content", b"environment content"

        mock_requests_get.side_effect = [get1, get2]

        clone('aakashns/metrics-example', version='3', fresh=False)

        os.chdir(dir)

        mock_get_gist.assert_called_with('aakashns/metrics-example', '3', False)
        mock_requests_get.assert_has_calls([
            call("https://storage.com/slug1"),
            call("https://storage.com/slug2")
        ])

        # Check that directory was not created
        assert "metrics-example" not in os.listdir()

        # Check that files were downloaded
        assert set(os.listdir()) == {"environment.yml", "metrics-example.ipynb"}


@mock.patch("jovian.utils.clone.get")
@mock.patch("jovian.utils.clone.get_gist")
def test_clone_overwrite(mock_get_gist, mock_requests_get):
    with temp_directory() as dir:
        mock_get_gist.return_value = FAKE_GIST

        get1, get2 = mock.Mock(), mock.Mock()
        get1.content, get2.content = b"notebook content", b"environment content"

        mock_requests_get.side_effect = [get1, get2] * 2

        clone('aakashns/metrics-example', version='3')
        os.chdir(dir)

        # Check that directory was created
        assert "metrics-example" in os.listdir()

        clone('aakashns/metrics-example', version='3', overwrite=True)
        os.chdir(dir)

        # Check that another directory was not created
        assert os.listdir() == ["metrics-example"]


@mock.patch("jovian.utils.clone.clone")
def test_pull(mock_clone):
    pull('aakashns/metrics-example', version=3)
    mock_clone.assert_called_with('aakashns/metrics-example', 3, fresh=False)


@mock.patch("jovian.utils.clone.rcfile_exists", return_value=False)
def test_pull_rcfile_does_not_exist(mock_rcfile_exists, capsys):
    pull()

    expected_result = """[jovian] Error: Could not detect '.jovianrc' file. Make sure you are running 'jovian pull' inside a directory cloned with 'jovian clone'."""
    captured = capsys.readouterr()
    assert captured.err.strip() == expected_result.strip()


@contextmanager
def fake_rcdata():
    with temp_directory():
        data = dedent("""
                {
                    "notebooks": {
                        "metrics-example.ipynb": {
                        "slug": "aakashns/metrics-example"
                        },
                        "jovian-tutorial.ipynb" : {
                        "slug": "aakashns/jovian-tutorial"
                        }
                    }
                }""").strip()

        with open('.jovianrc', 'w') as f:
            f.write(data)

        yield


@mock.patch("jovian.utils.clone.clone")
def test_pull_get_latest_notebooks(mock_clone):
    with fake_rcdata():
        pull()

        calls = [
            call('aakashns/metrics-example', None, fresh=False),
            call('aakashns/jovian-tutorial', None, fresh=False),
        ]
        mock_clone.assert_has_calls(calls, any_order=True)



from jovian.utils.error import ApiError, CondaError, ConfigError
import pytest


@pytest.mark.parametrize(
    "Error",
    [
        CondaError,
        ApiError,
        ConfigError
    ]
)
def test_error(Error):
    msg = 'This is a error'
    e = Error(msg)
    assert e.args == (msg,)



from jovian.utils.logger import log


def test_log(capsys):
    log("This is a log")
    captured = capsys.readouterr()
    assert captured.out.strip() == "[jovian] This is a log"


def test_log_error(capsys):
    log("This is an error", error=True)
    captured = capsys.readouterr()
    assert captured.err.strip() == "[jovian] Error: This is an error"


def test_log_warn(capsys):
    log("This is a warning", warn=True)
    captured = capsys.readouterr()
    assert captured.out.strip() == "[jovian] This is a warning"



import functools
import os
import shutil
from contextlib import contextmanager
from textwrap import dedent
from unittest import mock
from unittest.mock import ANY, call

import pytest

from jovian.tests.resources.shared import fake_creds, fake_records, mock_git_repo, temp_directory, touch
from jovian.utils.commit import (_attach_file, _attach_files, _attach_records, _capture_environment, _list_ipynb_files,
                                 _parse_filename, _parse_project, _perform_git_commit, commit, commit_path)
from jovian.utils.error import CondaError


def mock_get_gist(project):
    responses = {
        "f67108fc906341d8b15209ce88ebc3d2": {
            "slug": "f67108fc906341d8b15209ce88ebc3d2",
            "title": "demo-notebook",
            "owner": {
                "avatar": "https://api-staging.jovian.ai/api/user/rohit/avatar",
                "id": 47,
                "name": "Rohit Sanjay",
                "username": "rohit"
            }
        },
        "rohit/time-series": {
            "slug": "f67108fc906341d8b15209ce88ebc3d2",
            "title": "demo-notebook",
            "owner": {
                "avatar": "https://api-staging.jovian.ai/api/user/rohit/avatar",
                "id": 47,
                "name": "Rohit Sanjay",
                "username": "rohit"
            }
        },
        "rohit/time-series-new":  {
            "slug": None,
            "title": "demo-notebook",
            "owner": {
                "avatar": "https://api-staging.jovian.ai/api/user/rohit/avatar",
                "id": 47,
                "name": "Rohit Sanjay",
                "username": "rohit"
            },
        }
    }

    return responses[project]


@pytest.mark.parametrize(
    "filename, expected_result",
    [
        (None, 'fake-script.py'),
        ('script.py', 'script.py'),
        ('script', 'script.py')
    ]
)
@mock.patch("jovian.utils.commit.get_script_filename", return_value='fake-script.py')
@mock.patch("jovian.utils.commit.in_script", return_value=True)
def test_parse_filename_in_script(mock_in_script, mock_get_script_filename, filename, expected_result):
    assert _parse_filename(filename=filename) == expected_result


@pytest.mark.parametrize(
    "filename, expected_result",
    [
        (None, 'fake-notebook.ipynb'),
        ('notebook.ipynb', 'notebook.ipynb'),
        ('notebook', 'notebook.ipynb')
    ]
)
@mock.patch("jovian.utils.commit.in_script", return_value=False)
@mock.patch("jovian.utils.commit.get_notebook_name", return_value='fake-notebook.ipynb')
@mock.patch("jovian.utils.commit.in_notebook", return_value=True)
def test_parse_filename_in_notebook(mock_in_notebook, mock_get_script_name, mock_in_script, filename, expected_result):
    assert _parse_filename(filename=filename) == expected_result


@pytest.mark.parametrize(
    "get_gist_side_effect, get_gist_access_return_value, args, expected_result",
    [
        (
            mock_get_gist,
            {"write": True},
            {
                "project": None,
                "filename": "file.ipynb",
                "new_project": False,
            },
            ("demo-notebook", "f67108fc906341d8b15209ce88ebc3d2")
        ),
        (
            mock_get_gist,
            {"write": True},
            {
                "project": "rohit/time-series",
                "filename": "file.ipynb",
                "new_project": False,
            },
            ("demo-notebook", "f67108fc906341d8b15209ce88ebc3d2")
        ),
        (
            mock_get_gist,
            {"write": True},
            {
                "project": "time-series",
                "filename": "file.ipynb",
                "new_project": True,
            },
            ("demo-notebook", "f67108fc906341d8b15209ce88ebc3d2")
        ),
        (
            [None],
            {"write": False},
            {
                "project": "time-series",
                "filename": "file.ipynb",
                "new_project": True,
            },
            ("time-series", None)
        ),
        (
            mock_get_gist,
            {"write": False},
            {
                "project": "time-series",
                "filename": "file.ipynb",
                "new_project": True,
            },
            ("demo-notebook", None)
        ),
        (
            mock_get_gist,
            {"write": False},
            {
                "project": "rohit/time-series-new",
                "filename": "file.ipynb",
                "new_project": True,
            },
            ("demo-notebook", None)
        ),
        (
            None,
            None,
            {
                "project": None,
                "filename": "file.ipynb",
                "new_project": True,
            },
            (None, None)
        ),

    ]
)
@mock.patch("jovian.utils.rcfile._current_slug", "f67108fc906341d8b15209ce88ebc3d2")
@mock.patch("jovian.utils.commit.api.get_current_user", return_value={'username': 'rohit'})
def test_parse_project(
        mock_get_current_user, get_gist_side_effect, get_gist_access_return_value, args, expected_result):
    with mock.patch("jovian.utils.commit.api.get_gist", side_effect=get_gist_side_effect), \
            mock.patch("jovian.utils.commit.api.get_gist_access", return_value=get_gist_access_return_value):
        with fake_creds():
            assert _parse_project(**args) == expected_result


@mock.patch("jovian.utils.rcfile._current_slug", None)
@mock.patch("jovian.utils.commit.get_notebook_slug", return_value="rohit/time-series-new")
@mock.patch("jovian.utils.commit.api.get_current_user", return_value={'username': 'rohit'})
@mock.patch("jovian.utils.commit.api.get_gist", side_effect=mock_get_gist)
@mock.patch("jovian.utils.commit.api.get_gist_access", return_value={"write": True})
def test_parse_project_from_rcfile(mock_get_gist_access, mock_get_gist, mock_get_current_user, mock_get_notebook_slug):
    with fake_creds():
        assert _parse_project(project=None, filename="file.ipynb", new_project=False) == ('demo-notebook', None)


@mock.patch("jovian.utils.commit.api.upload_file")
def test_attach_file(mock_upload_file):
    with temp_directory():
        os.system('touch tempfile.txt')

        _attach_file('tempfile.txt', 'fake_gist_slug', version=2)
        mock_upload_file.assert_called_with('fake_gist_slug', ('tempfile.txt', ANY), ANY, 2, False)


def test_attach_file_raises_error(capsys):
    with temp_directory():
        _attach_file('tempfile.txt', 'fake_gist_slug', version=2)

        expected_result = "[jovian] Error: [Errno 2] No such file or directory: 'tempfile.txt' (tempfile.txt)"
        captured = capsys.readouterr()
        assert captured.err.strip() == expected_result


@mock.patch("jovian.utils.commit._attach_file")
@pytest.mark.parametrize(
    "attach_files_kwargs, extra_config, mock_calls",
    [
        (
            {"paths": []},
            {},
            []
        ),
        (
            {"paths": ['tempdir']},
            {},
            [
                call('tempdir/file.txt', 'fake_gist_slug', 2, False),
                call('tempdir/subdir/file1.txt', 'fake_gist_slug', 2, False)
            ]
        ),
        (
            {"paths": "tempdir"},
            {},
            [
                call('tempdir/file.txt', 'fake_gist_slug', 2, False),
                call('tempdir/subdir/file1.txt', 'fake_gist_slug', 2, False)
            ]
        ),
        (
            {"paths": ['tempdir/file.txt', 'tempdir/subdir', 'tempdir/doesnotexist.txt']},
            {},
            [
                call('tempdir/file.txt', 'fake_gist_slug', 2, False),
                call('tempdir/subdir/file1.txt', 'fake_gist_slug', 2, False)
            ]
        ),
        (
            {"paths": []},
            {
                "DEFAULT_CONFIG": {
                    "UPLOAD_WORKING_DIRECTORY": True
                }
            },
            [
                call('notebook.ipynb', 'fake_gist_slug', 2, False),
                call('valid/file.txt', 'fake_gist_slug', 2, False),
                call('valid/file.ipynb', 'fake_gist_slug', 2, False),
                call('valid/file.tsv', 'fake_gist_slug', 2, False),
                call('valid/file.yaml', 'fake_gist_slug', 2, False),
                call('valid/file.yml', 'fake_gist_slug', 2, False),
                call('valid/file.py', 'fake_gist_slug', 2, False),
                call('valid/file.csv', 'fake_gist_slug', 2, False),
                call('tempdir/file.txt', 'fake_gist_slug', 2, False),
                call('tempdir/subdir/file1.txt', 'fake_gist_slug', 2, False)
            ]
        ),
        (
            {"paths": [], "output": True},
            {},
            []
        ),
        (
            {"paths": [], "output": True},
            {
                "DEFAULT_CONFIG": {
                    "UPLOAD_WORKING_DIRECTORY": True
                }
            },
            []
        ),
        (
            {"paths": [], "exclude_files": "notebook.ipynb"},
            {
                "DEFAULT_CONFIG": {
                    "UPLOAD_WORKING_DIRECTORY": True
                }
            },
            [
                call('valid/file.txt', 'fake_gist_slug', 2, False),
                call('valid/file.ipynb', 'fake_gist_slug', 2, False),
                call('valid/file.tsv', 'fake_gist_slug', 2, False),
                call('valid/file.yaml', 'fake_gist_slug', 2, False),
                call('valid/file.yml', 'fake_gist_slug', 2, False),
                call('valid/file.py', 'fake_gist_slug', 2, False),
                call('valid/file.csv', 'fake_gist_slug', 2, False),
                call('tempdir/file.txt', 'fake_gist_slug', 2, False),
                call('tempdir/subdir/file1.txt', 'fake_gist_slug', 2, False)
            ]
        ),
        (
            {
                "paths": [],
                "exclude_files": ["notebook.ipynb", "valid/file.txt", "tempdir/file.txt", "nonexistentfile.txt"]
            },
            {
                "DEFAULT_CONFIG": {
                    "UPLOAD_WORKING_DIRECTORY": True
                }
            },
            [
                call('valid/file.ipynb', 'fake_gist_slug', 2, False),
                call('valid/file.tsv', 'fake_gist_slug', 2, False),
                call('valid/file.yaml', 'fake_gist_slug', 2, False),
                call('valid/file.yml', 'fake_gist_slug', 2, False),
                call('valid/file.py', 'fake_gist_slug', 2, False),
                call('valid/file.csv', 'fake_gist_slug', 2, False),
                call('tempdir/subdir/file1.txt', 'fake_gist_slug', 2, False)
            ]
        ),
    ]
)
def test_attach_files(mock_attach_file, attach_files_kwargs, extra_config, mock_calls, capsys):
    with fake_creds(extra=extra_config):

        os.makedirs('tempdir/subdir')
        os.system('touch tempdir/file.txt && touch tempdir/subdir/file1.txt')

        # current notebook
        os.system('touch notebook.ipynb')

        # valid files
        os.mkdir('valid')
        valid_files = ["file.txt", "file.py", "file.csv", "file.yaml", "file.yml", "file.ipynb", "file.tsv"]
        for file in valid_files:
            os.system("touch valid/{}".format(file))

        # invalid files
        os.mkdir('invalid')
        invalid_files = ["file.file", "no_extension", "file.pyc"]
        for file in invalid_files:
            os.system("touch invalid/{}".format(file))

        _attach_files(gist_slug='fake_gist_slug', version=2, **attach_files_kwargs)

        mock_attach_file.assert_has_calls(mock_calls, any_order=True)


@mock.patch("jovian.utils.commit.upload_conda_env")
def test_capture_environment(mock_upload_conda_env):
    with fake_creds():
        _capture_environment('conda', 'fake_gist_slug', 2)

        mock_upload_conda_env.assert_called_with('fake_gist_slug', 2)


@pytest.mark.parametrize(
    "environment, env_config, mock_upload",
    [
        ("auto", "conda", "jovian.utils.commit.upload_conda_env"),
        ("auto", "pip", "jovian.utils.commit.upload_pip_env"),
    ]
)
def test_capture_environment_from_config(environment, env_config, mock_upload):
    with fake_creds(extra={"DEFAULT_CONFIG": {"environment": env_config}}), mock.patch(mock_upload) as mock_upload:
        _capture_environment(environment, 'fake_gist_slug', 2)
        mock_upload.assert_called_with('fake_gist_slug', 2)


@mock.patch("jovian.utils.commit.upload_pip_env")
@mock.patch("jovian.utils.commit.upload_conda_env")
def test_capture_environment_from_config_none(mock_upload_conda_env, mock_upload_pip_env):
    with fake_creds(extra={"DEFAULT_CONFIG": {"environment": None}}):
        _capture_environment("auto", "fake_gist_slug", 2)

        mock_upload_conda_env.assert_not_called()
        mock_upload_pip_env.assert_not_called()


@mock.patch("jovian.utils.commit.upload_pip_env")
@mock.patch("jovian.utils.commit.upload_conda_env")
def test_capture_environment_conda_error_and_pip_exception(mock_upload_conda_env, mock_upload_pip_env, capsys):

    def conda_error_raiser(*args, **kwargs):
        raise CondaError('fake conda error')

    def pip_exception_raiser(*args, **kwargs):
        raise Exception('fake pip error')

    mock_upload_conda_env.side_effect = conda_error_raiser
    mock_upload_pip_env.side_effect = pip_exception_raiser

    _capture_environment('auto', 'fake_gist_slug', 2)

    mock_upload_conda_env.assert_called_with('fake_gist_slug', 2)
    mock_upload_pip_env.assert_called_with('fake_gist_slug', 2)

    expected_result = dedent("""
            [jovian] Error: fake conda error
            [jovian] Error: fake pip error
            """).strip()
    captured = capsys.readouterr()
    assert captured.err.strip() == expected_result


@mock.patch("jovian.utils.commit.log_git")
def test_perform_git_commit(mock_log_git, capsys):
    with mock_git_repo():
        os.system("echo \"hello world\" >> file.txt")

        _perform_git_commit(filename='file.txt', git_commit=True, git_message='added file.txt')

        # Assert that commit took place
        assert os.popen('git show -s --format=%s HEAD').read().strip() == 'added file.txt'

        # Assert that it was logged
        expected_result = "[jovian] Git repository identified. Performing git commit..."
        captured = capsys.readouterr()
        assert captured.out.strip() == expected_result

        mock_log_git.assert_called_with(
            {
                'repository': 'https://github.com/JovianML/mock_repo',
                'commit': ANY,
                'filename': 'file.txt',
                'path': '.',
                'branch': 'master'
            },
            verbose=False
        )


@mock.patch("jovian.utils.commit.api.post_records")
def test_attach_records(mock_api_post_records, capsys):
    with fake_records():
        _attach_records('fake_gist_slug', 2)

        mock_api_post_records.assert_called_with(
            'fake_gist_slug',
            ['fake_slug_metrics_1', 'fake_slug_metrics_2', 'fake_slug_hyperparams_1', 'fake_slug_hyperparams_2'],
            2
        )

        expected_result = "[jovian] Attaching records (metrics, hyperparameters, dataset etc.)"
        captured = capsys.readouterr()
        assert captured.out.strip() == expected_result


def test_list_ipynb_files(tmpdir, capsys):
    folder1 = tmpdir.mkdir("sub")
    folder2 = tmpdir.mkdir("sub2")
    file1 = folder1.join("test1.ipynb").ensure(file=True)
    file2 = folder1.join("test2.ipynb").ensure(file=True)
    file3 = folder1.join("test3.py").ensure(file=True)

    assert len(tmpdir.listdir()) == 2
    assert len(_list_ipynb_files(str(file1))) == 1
    assert len(_list_ipynb_files(str(file3))) == 0
    assert len(_list_ipynb_files(str(folder1))) == 2
    assert len(_list_ipynb_files(str(folder2))) == 0


@pytest.mark.parametrize(
    "ipynb_files, confirm, expected_files", [
        ([], True, []),
        ([], False, []),
        (["test1.ipynb"], True, ["test1.ipynb"]),
        (["test1.ipynb"], False, []),
        (["test1.ipynb", "test2.ipynb"], True, ["test1.ipynb", "test2.ipynb"]),
        (["test1.ipynb", "test2.ipynb"], False, []),
        (["test{}.ipynb".format(i) for i in range(0, 20)], True, ["test{}.ipynb".format(i) for i in range(0, 20)]),
        (["test{}.ipynb".format(i) for i in range(0, 20)], False, []),
        (["test{}.ipynb".format(i) for i in range(0, 50)], True, []),
        (["test{}.ipynb".format(i) for i in range(0, 50)], False, [])
    ]
)
@mock.patch("time.sleep")
@mock.patch("jovian.utils.commit.commit")
def test_commit_path(mock_commit, mock_sleep, ipynb_files, confirm, expected_files):
    with mock.patch("jovian.utils.commit._list_ipynb_files", return_value=ipynb_files) as mock_func:
        with mock.patch("click.confirm", return_value=confirm) as mock_confirm:
            commit_path(path="notebook", environment=None, is_cli=True)
            mock_commit.assert_has_calls([call(filename=f, environment=None, is_cli=True) for f in expected_files])


@pytest.mark.parametrize(
    "commit_kwargs, expected_result",
    [({"secret": True},
      """[jovian] Error: "secret" is deprecated. Use "privacy" instead (allowed options: "public", "private", "secret", "auto")"""),
     ({"nb_filename": 'file'},
      """[jovian] Error: "nb_filename" is deprecated. Use "filename" instead"""),
     ({"env_type": "conda"},
      """[jovian] Error: "env_type" is deprecated. Use "environment" instead"""),
     ({"capture_env": False},
      """[jovian] Error: "catpure_env" is deprecated. Use "environment=None" instead"""),
     ({"notebook_id": "fake_notebook_id"},
      """[jovian] Error: "notebook_id" is deprecated. Use "project" instead."""),
     ({"create_new": True},
      """[jovian] Error: "create_new" is deprecated. Use "new_project" instead."""),
     ({"artifacts": ['file1', 'file2']},
      """[jovian] Error: "artifacts" is deprecated. Use "outputs" instead"""), ])
@mock.patch("jovian.utils.commit.in_notebook", return_value=False)
@mock.patch("jovian.utils.commit.in_script", return_value=False)
def test_commit_deprecated_args(mock_in_script, mock_in_notebook, commit_kwargs, expected_result, capsys):

    commit(message="this is the first commit", **commit_kwargs)
    captured = capsys.readouterr()
    assert captured.err.strip() == \
        expected_result.strip() + """\n[jovian] Error: Failed to detect Jupyter notebook or Python script. Skipping.."""


@mock.patch("jovian.utils.commit.save_notebook", return_value=None)
@mock.patch("jovian.utils.commit._parse_filename", return_value=None)
@mock.patch("jovian.utils.commit.in_notebook", return_value=True)
@mock.patch("jovian.utils.commit.in_script", return_value=False)
def test_commit_in_notebook_filename_none(
        mock_in_script, mock_in_notebook, mock_parse_filename, mock_save_notebook, capsys):

    commit('initial commit')
    expected_result_out = dedent("""
    [jovian] Attempting to save notebook..""")
    expected_result_err = dedent("""
        [jovian] Error: Failed to detect notebook filename. Please provide the correct notebook filename as the "filename" argument to "jovian.commit".""")
    captured = capsys.readouterr()
    assert captured.out.strip() == expected_result_out.strip()
    assert captured.err.strip() == expected_result_err.strip()


@mock.patch("jovian.utils.commit.os.path.exists", return_value=False)
@mock.patch("jovian.utils.commit._parse_filename", return_value='file')
@mock.patch("jovian.utils.commit.in_notebook", return_value=False)
@mock.patch("jovian.utils.commit.in_script", return_value=True)
def test_commit_file_does_not_exist(
        mock_in_script, mock_in_notebook, mock_parse_filename, mock_os_path_exists, capsys):

    commit('initial commit')

    expected_result = """[jovian] Error: The detected/provided file "file" does not exist. Please provide the correct notebook filename as the "filename" argument to "jovian.commit"."""
    captured = capsys.readouterr()
    assert captured.err.strip() == expected_result.strip()


def mock_create_gist_simple(*args, **kwargs):
    data = {
        "slug": "fake_gist_slug",
        "title": "demo-notebook",
        "owner": {
                "avatar": "https://api-staging.jovian.ai/api/user/rohit/avatar",
                "id": 47,
                "name": "Rohit Sanjay",
                "username": "rohit"
        },
        "version": 2
    }
    return data


def patch_all(f):
    @mock.patch("jovian.utils.commit._attach_records")
    @mock.patch("jovian.utils.commit._perform_git_commit")
    @mock.patch("jovian.utils.commit._attach_files")
    @mock.patch("jovian.utils.commit._capture_environment")
    @mock.patch("jovian.utils.commit.set_notebook_slug")
    @mock.patch("jovian.utils.commit.read_webapp_url", return_value='https://staging.jovian.ml/')
    @mock.patch("jovian.utils.commit.api.create_gist_simple", side_effect=mock_create_gist_simple)
    @mock.patch("jovian.utils.commit._parse_project", return_value=('fake_project_title', 'fake_project_id'))
    @mock.patch("jovian.utils.commit.os.path.exists", return_value=True)
    @mock.patch("jovian.utils.commit._parse_filename", return_value='file')
    @mock.patch("jovian.utils.commit.in_notebook", return_value=False)
    @mock.patch("jovian.utils.commit.in_script", return_value=True)
    @functools.wraps(f)
    def functor(*args, **kwargs):
        return f(*args, **kwargs)
    return functor


@patch_all
def test_commit(mock_in_script,
                mock_in_notebook,
                mock_parse_filename,
                mock_os_path_exists,
                mock_parse_project,
                mock_create_gist_simple,
                mock_read_webapp_url,
                mock_set_notebook_slug,
                mock_capture_environment,
                mock_attach_files,
                mock_perform_git_commit,
                mock_attach_records,
                capsys):

    returned_value = commit('initial commit', files=['file1', 'file2', 'file3'], privacy='secret',
                            environment='conda', outputs=['model.h5', 'gen.csv'])

    mock_create_gist_simple.assert_called_with(
        'file', 'fake_project_id', 'secret', 'fake_project_title', 'initial commit')

    mock_set_notebook_slug.assert_called_with('file', 'fake_gist_slug')

    mock_capture_environment.assert_called_with('conda', 'fake_gist_slug', 2)

    _attach_files_calls = [call(['file1', 'file2', 'file3'], 'fake_gist_slug', 2, exclude_files='file'),
                           call(['model.h5', 'gen.csv'], 'fake_gist_slug', 2, output=True)]
    mock_attach_files.assert_has_calls(_attach_files_calls)

    mock_perform_git_commit.assert_called_with('file', False, 'initial commit')

    mock_attach_records.assert_called_with('fake_gist_slug', 2)

    expected_common_result = "[jovian] Committed successfully! https://staging.jovian.ml/rohit/demo-notebook"
    captured = capsys.readouterr()
    assert captured.out.strip() == expected_common_result.strip()
    assert returned_value == "https://staging.jovian.ml/rohit/demo-notebook"



from unittest import TestCase, mock
import pytest
from jovian.utils.script import get_script_filename, in_script


def test_get_script_filename_normal():
    import __main__
    __main__.__file__ = 'sample.py'

    expected_result = 'sample.py'
    assert get_script_filename() == expected_result


@mock.patch('builtins.__import__', return_value=ImportError('fake import error'))
def test_get_script_filename_exception(mock_import):

    expected_result = None
    assert get_script_filename() == expected_result


@pytest.mark.parametrize(
    "filename, expected_result",
    [
        ('sample.py', True),
        ('sample.txt', False),
        (None, False)
    ]
)
def test_in_script(filename, expected_result):
    import __main__
    __main__.__file__ = filename

    assert in_script() == expected_result



import os
from unittest import TestCase, mock
from uuid import UUID

import pytest

from jovian.tests.resources.shared import MockResponse, fake_creds
from jovian.utils import credentials
from jovian.utils.credentials import (API_TOKEN_KEY, GUEST_TOKEN_KEY, _generate_guest_key, _read_or_generate_guest_key,
                                      _u, _validate_guest_key, config_exists, creds_exist, ensure_org, get_api_key,
                                      get_creds_path, get_guest_key, init_config, purge_api_key, purge_config,
                                      purge_cred_key, purge_creds, read_api_key_opt, read_api_url, read_cred,
                                      read_creds, read_or_request_api_key, read_org_id, read_webapp_url,
                                      request_api_key, request_org_id, validate_api_key, write_api_key, write_api_url,
                                      write_cred, write_creds, write_guest_key, write_org_id, write_webapp_url)
from jovian.utils.error import ApiError, ConfigError

try:
    # Python 3
    from json.decoder import JSONDecodeError
except ImportError:
    # Python 2
    JSONDecodeError = ValueError


def test_config_exits_false():
    with fake_creds():
        purge_config()
        assert config_exists() == False


def test_config_exists_true():
    with fake_creds():
        assert config_exists() == True


def test_purge_config():
    with fake_creds():
        os.makedirs(credentials.CONFIG_DIR, exist_ok=True)
        assert config_exists() == True
        purge_config()
        assert config_exists() == False


def test_get_creds_path():
    with fake_creds() as dir:
        assert get_creds_path() == os.path.join(dir, '.jovian/credentials.json')


def test_init_config():
    with fake_creds() as dir:
        init_config()
        assert os.path.exists(os.path.join(dir, '.jovian')) == True


def test_purge_creds():
    with fake_creds() as dir:
        assert os.path.exists(os.path.join(dir, '.jovian/credentials.json')) == True
        purge_creds()
        assert os.path.exists(os.path.join(dir, '.jovian/credentials.json')) == False


def test_purge_api_key():
    with fake_creds():
        assert read_cred(API_TOKEN_KEY) == 'fake_api_key'

        purge_api_key()

        with pytest.raises(KeyError):
            read_cred(API_TOKEN_KEY)


def test_read_creds_no_creds_folder():
    with fake_creds():
        purge_creds()
        assert read_creds() == {}


def test_read_creds_folder_exists():
    with fake_creds():
        expected_result = {"WEBAPP_URL": "https://staging.jovian.ml/",
                           "GUEST_KEY": "b6538d4dfde04fcf993463a828a9cec6",
                           "ORG_ID": "staging",
                           "API_URL": "https://api-staging.jovian.ai",
                           "API_KEY": "fake_api_key"}
        assert read_creds() == expected_result


def mock_json_load(*args, **kwargs):
    raise ValueError('fake value error')


@mock.patch("jovian.utils.credentials.purge_creds", return_value=None)
@mock.patch("json.load", side_effect=mock_json_load)
def test_read_creds_with_value_error(mock_json_load, mock_purge_creds):
    with fake_creds():
        assert read_creds() == {}


def test_creds_exist_true():
    with fake_creds():
        assert creds_exist() == True


def test_creds_exist_false():
    with fake_creds():
        purge_api_key()

        assert creds_exist() == False


def test_read_cred_no_default():
    with fake_creds():
        assert read_cred('ORG_ID') == 'staging'


def test_read_cred_with_default():
    with fake_creds():
        assert read_cred('FAKE_ID', 'default_value') == 'default_value'


def test_write_creds():
    with fake_creds():
        creds = {"WEBAPP_URL": "https://staging.jovian.ml/",
                 "GUEST_KEY": "b6538d4dfde04fcf993463a828a9cec6",
                 "ORG_ID": "staging",
                 "API_URL": "https://api-staging.jovian.ai"}
        write_creds(creds)

        assert read_creds() == creds


def test_write_cred():
    with fake_creds():
        creds = {"WEBAPP_URL": "https://staging.jovian.ml/",
                 "GUEST_KEY": "b6538d4dfde04fcf993463a828a9cec6",
                 "ORG_ID": "staging",
                 "API_URL": "https://api-staging.jovian.ai"}
        write_creds(creds)

        write_cred('FAKE_KEY', 'fake_value')

        expected_result = {"WEBAPP_URL": "https://staging.jovian.ml/",
                           "GUEST_KEY": "b6538d4dfde04fcf993463a828a9cec6",
                           "ORG_ID": "staging",
                           "API_URL": "https://api-staging.jovian.ai",
                           "FAKE_KEY": "fake_value"}
        assert read_creds() == expected_result


def test_write_cred_already_exists():
    with fake_creds():
        creds = {"WEBAPP_URL": "https://staging.jovian.ml/",
                 "GUEST_KEY": "b6538d4dfde04fcf993463a828a9cec6",
                 "ORG_ID": "staging",
                 "API_URL": "https://api-staging.jovian.ai"}
        write_creds(creds)

        write_cred('ORG_ID', 'staging')

        expected_result = creds
        assert read_creds() == expected_result


def test_purge_cred_key():
    with fake_creds():
        creds = {"WEBAPP_URL": "https://staging.jovian.ml/",
                 "GUEST_KEY": "b6538d4dfde04fcf993463a828a9cec6",
                 "ORG_ID": "staging",
                 "API_URL": "https://api-staging.jovian.ai"}
        write_creds(creds)

        purge_cred_key('GUEST_KEY')

        expected_result = {"WEBAPP_URL": "https://staging.jovian.ml/",
                           "ORG_ID": "staging",
                           "API_URL": "https://api-staging.jovian.ai"}

        assert read_creds() == expected_result


def test_write_api_url():
    with fake_creds():
        write_api_url("https://fake_api.jovian.ai/")

        from jovian.utils.credentials import API_URL_KEY
        assert read_cred(API_URL_KEY) == "https://fake_api.jovian.ai/"


def test_read_api_url():
    with fake_creds():
        write_api_url("https://fake_api.jovian.ai/")
        assert read_api_url() == "https://fake_api.jovian.ai/"


def test_write_org_id():
    with fake_creds():
        write_org_id("fake_org_id")

        from jovian.utils.credentials import ORG_ID_KEY
        assert read_cred(ORG_ID_KEY) == "fake_org_id"


def test_read_org_id():
    with fake_creds():
        write_org_id("fake_org_id")
        assert read_org_id() == "fake_org_id"


def test_write_webapp_url():
    with fake_creds():
        write_webapp_url("http://fake-webapp-url.ai/")

        from jovian.utils.credentials import WEBAPP_URL_KEY
        assert read_cred(WEBAPP_URL_KEY) == "http://fake-webapp-url.ai/"


def test_read_webapp_url():
    with fake_creds():
        write_webapp_url("http://fake-webapp-url.ai/")
        assert read_webapp_url() == "http://fake-webapp-url.ai/"


@mock.patch("click.prompt", return_value="fake_org_id")
def test_request_org_id(mock_prompt):
    assert request_org_id() == "fake_org_id"


def mock_requests_get(url, *args, **kwargs):
    if url == 'https://jovian.ml/config.json':
        data = {"API_URL": "https://api.jovian.ai"}

        return MockResponse(data, status_code=200)

    elif url == 'https://fakecompany.jovian.ml/config.json':

        return MockResponse({"msg": "Request failed"}, status_code=500, text="Fake internal server error")

    elif url == 'https://jsonerror.jovian.ml/config.json':
        data = {"API_URL": "https://api.jovian.ai"}
        res = MockResponse(data, status_code=200, text="response of fake json decode error")
        res.json = json_decode_error_raiser

        return res

    elif url == 'https://no-api-key.jovian.ml/config.json':
        data = {}

        return MockResponse(data, status_code=200, text="response with no api key")

    elif url == 'https://api-staging.jovian.ai/user/profile':
        key = kwargs['headers']['Authorization']
        if key == 'Bearer fake_correct_auth_key':
            return MockResponse({'key': 'value'}, 200)

        return MockResponse({'key': 'value'}, 401)


def connection_error_raiser(*args, **kwargs):
    raise ConnectionError('fake connection error')


def json_decode_error_raiser(*args, **kwargs):
    raise JSONDecodeError('fake json decode error', 'credentials.json', 0)


@mock.patch("jovian.utils.credentials.is_flavor_pro", return_value=True)
def test_ensure_org_all_creds_exist(mock_is_flavor_pro):
    with fake_creds():
        assert ensure_org() == None


@mock.patch("requests.get", side_effect=mock_requests_get)
@mock.patch("jovian.utils.credentials.request_org_id", return_value="")
@mock.patch("jovian.utils.credentials.is_flavor_pro", return_value=True)
def test_ensure_org_some_creds_exist_default_org_id(
        mock_is_flavor_pro, mock_request_org_id, mock_requests_get):
    with fake_creds():
        # setUp
        creds = {"WEBAPP_URL": "https://staging.jovian.ml/",
                 "GUEST_KEY": "b6538d4dfde04fcf993463a828a9cec6",
                 "API_URL": "https://api-staging.jovian.ai"}
        write_creds(creds)

        ensure_org()

        assert read_api_url() == "https://api.jovian.ai"
        assert read_org_id() == "public"
        assert read_webapp_url() == "https://jovian.ml/"


@pytest.mark.parametrize(
    "get_side_effect, request_org_id, creds, msg",
    [
        (
            connection_error_raiser,
            "fakecompany",
            {"WEBAPP_URL": "https://staging.jovian.ml/",
             "API_URL": "https://api-staging.jovian.ai"},
            "Failed to connect to https://fakecompany.jovian.ml/ . Please verify your organization ID and " +
            "ensure you are connected to the internet."
        ),
        (
            mock_requests_get,
            "fakecompany",
            {"WEBAPP_URL": "https://staging.jovian.ml/",
             "API_URL": "https://api-staging.jovian.ai"},
            "Request to retrieve configuration file https://fakecompany.jovian.ml/config.json failed with " +
            "status_code 500 . Looks like there's something wrong with your setup.",
        ),
        (
            mock_requests_get,
            "jsonerror",
            {"WEBAPP_URL": "https://staging.jovian.ml/",
             "API_URL": "https://api-staging.jovian.ai"},
            "Failed to parse JSON configuration file from https://jsonerror.jovian.ml/config.json",
        ),
        (
            mock_requests_get,
            "no-api-key",
            {"WEBAPP_URL": "https://staging.jovian.ml/"},
            "Failed to extract API_URL from JSON configuration file https://no-api-key.jovian.ml/config.json"
        ),
    ]
)
@mock.patch("jovian.utils.credentials.is_flavor_pro", return_value=True)
def test_ensure_org_pro_raises_error(mock_is_flavor_pro, get_side_effect, request_org_id, creds, msg):
    with fake_creds(), \
            mock.patch("jovian.utils.credentials.request_org_id", return_value=request_org_id), \
            mock.patch("requests.get", side_effect=get_side_effect):

        write_creds(creds)

        with pytest.raises(ConfigError) as context:
            ensure_org()

        assert msg in str(context.value)


def test_write_api_key():
    with fake_creds():
        write_api_key("fake_api_key")

        from jovian.utils.credentials import API_TOKEN_KEY
        assert read_cred(API_TOKEN_KEY) == "fake_api_key"


@pytest.mark.parametrize(
    "api_key, expected_result",
    [
        ("fake_correct_auth_key", True),
        ("fake_invalid_auth_key", False),
    ]
)
@mock.patch("requests.get", side_effect=mock_requests_get)
def test_validate_api_key(mock_requests_get, api_key, expected_result):
    with fake_creds():
        assert validate_api_key(api_key) == expected_result


def test_write_guest_key():
    with fake_creds():
        write_guest_key("fake_guest_key")

        from jovian.utils.credentials import GUEST_TOKEN_KEY
        assert read_cred(GUEST_TOKEN_KEY) == "fake_guest_key"


@mock.patch("click.prompt", return_value="fake_api_key")
def test_request_api_key(mock_prompt):
    assert request_api_key() == "fake_api_key"


def test_read_api_key_opt():
    with fake_creds():
        assert read_api_key_opt() == ('fake_api_key', 'read')


def test_read_api_key_opt_none():
    with fake_creds():
        purge_api_key()

        assert read_api_key_opt() == (None, 'read')


def test_read_or_request_api_key():
    with fake_creds():
        assert read_or_request_api_key() == ('fake_api_key', 'read')


@mock.patch("click.prompt", return_value="fake_api_key")
def test_read_or_request_api_key_none(mock_prompt):
    with fake_creds():
        purge_api_key()

        assert read_or_request_api_key() == ('fake_api_key', 'request')


@mock.patch("uuid.uuid4", return_value=UUID('b66406dc-02c3-471b-ac27-d923fb4c6b1e'))
def test_generate_guest_key(mock_uuid4):
    assert _generate_guest_key() == 'b66406dc02c3471bac27d923fb4c6b1e'


def test_read_or_generate_guest_key():
    with fake_creds():
        assert _read_or_generate_guest_key() == "b6538d4dfde04fcf993463a828a9cec6"


@mock.patch("uuid.uuid4", return_value=UUID('b66406dc-02c3-471b-ac27-d923fb4c6b1e'))
def test_read_or_generate_guest_key_generate(mock_uuid4):
    with fake_creds():
        purge_cred_key(GUEST_TOKEN_KEY)
        assert _read_or_generate_guest_key() == "b66406dc02c3471bac27d923fb4c6b1e"


def test_validate_guest_key():
    assert _validate_guest_key("b6538d4dfde04fcf993463a828a9cec6") == True
    assert _validate_guest_key("fake-uuid-key") == False


def test_get_guest_key():
    with fake_creds():
        assert get_guest_key() == "b6538d4dfde04fcf993463a828a9cec6"


@mock.patch("uuid.uuid4", return_value=UUID('b66406dc-02c3-471b-ac27-d923fb4c6b1e'))
def test_get_guest_key_generate_key(mock_uuid4):
    with fake_creds():
        creds = {"WEBAPP_URL": "https://staging.jovian.ml/",
                 "ORG_ID": "staging",
                 "API_URL": "https://api-staging.jovian.ai"}
        write_creds(creds)

        assert get_guest_key() == "b66406dc02c3471bac27d923fb4c6b1e"


@mock.patch("jovian.utils.credentials.validate_api_key", return_value=True)
def test_get_api_key(mock_validate_api_key):
    with fake_creds():
        creds = {"WEBAPP_URL": "https://staging.jovian.ml/",
                 "ORG_ID": "staging",
                 "API_KEY": "fake_api_key",
                 "API_URL": "https://api-staging.jovian.ai"}
        write_creds(creds)

        assert get_api_key() == "fake_api_key"


@mock.patch("click.prompt", return_value="fake_api_key")
@mock.patch("jovian.utils.credentials.validate_api_key", return_value=False)
def test_get_api_key_api_error(mock_validate_api_key, mock_prompt):
    with fake_creds():
        creds = {"WEBAPP_URL": "https://staging.jovian.ml/",
                 "ORG_ID": "staging",
                 "API_URL": "https://api-staging.jovian.ai"}
        write_creds(creds)

        with pytest.raises(ApiError):
            get_api_key()


@mock.patch("click.prompt", return_value="fake_api_key")
@mock.patch("jovian.utils.credentials.validate_api_key", return_value=True)
def test_get_api_key_request_once(mock_validate_api_key, mock_prompt):
    with fake_creds():
        creds = {"WEBAPP_URL": "https://staging.jovian.ml/",
                 "ORG_ID": "staging",
                 "API_URL": "https://api-staging.jovian.ai"}
        write_creds(creds)

        assert get_api_key() == "fake_api_key"



import json
import os
import shutil
from contextlib import contextmanager
from unittest import TestCase, mock

import pytest

from jovian.tests.resources.shared import temp_directory
from jovian.utils.constants import RC_FILENAME
from jovian.utils.rcfile import (get_cached_slug, get_notebook_slug, get_rcdata, make_rcdata, rcfile_exists,
                                 reset_notebook_slug, save_rcdata, set_notebook_slug)

_data = {
    "notebooks": {
        "Testing Jovian.ipynb": {
            "slug": "46bd9a3f87e74de0baf8a6f0b60a8df9"
        }
    }
}


@contextmanager
def fake_rc():
    with temp_directory():
        with open(RC_FILENAME, 'w') as f:
            json.dump(_data, f, indent=2)

        yield


def test_rcfile_does_not_exist():
    with temp_directory():
        assert not rcfile_exists()


@pytest.mark.parametrize(
    "data, expected_result",
    [
        (_data, _data),
        (None, {
            "notebooks": {}
        })
    ]
)
def test_save_rcdata(data, expected_result):
    with temp_directory():
        save_rcdata(data=data)

        with open(RC_FILENAME, 'r') as f:
            assert json.load(f) == expected_result


def test_get_rcdata():
    with temp_directory():
        expected_result = {
            "notebooks": {}
        }
        assert get_rcdata() == expected_result

    with fake_rc():
        assert get_rcdata() == _data


@pytest.mark.parametrize(
    "filename, expected_result",
    [
        ("Testing Jovian.ipynb", "46bd9a3f87e74de0baf8a6f0b60a8df9"),
        ("Testing Jovian123.ipynb", None)
    ]
)
def test_get_notebook_slug_notebook_present(filename, expected_result):
    with fake_rc():
        assert get_notebook_slug(filename) == expected_result


def test_set_notebook_slug():
    with fake_rc():
        filename = "Testing Jovian 2.ipynb"
        slug = "46bd9a3f87e74de0baf8a6f0b60a8df9"

        expected_result = {
            "notebooks": {
                "Testing Jovian.ipynb": {
                    "slug": "46bd9a3f87e74de0baf8a6f0b60a8df9"
                },
                "Testing Jovian 2.ipynb": {
                    "slug": "46bd9a3f87e74de0baf8a6f0b60a8df9"
                }
            }
        }
        set_notebook_slug(filename, slug)
        assert get_rcdata() == expected_result


def test_make_rcdata():
    filename = "Testing Jovian.ipynb"
    slug = "46bd9a3f87e74de0baf8a6f0b60a8df9"
    expected_result = '{"notebooks": {"Testing Jovian.ipynb": {"slug": "46bd9a3f87e74de0baf8a6f0b60a8df9"}}}'

    assert make_rcdata(filename, slug) == expected_result


@mock.patch("jovian.utils.rcfile._current_slug", "f67108fc906341d8b15209ce88ebc3d2")
def test_get_cached_slug():
    assert get_cached_slug() == "f67108fc906341d8b15209ce88ebc3d2"


@mock.patch("jovian.utils.rcfile._current_slug", "f67108fc906341d8b15209ce88ebc3d2")
def test_reset_notebook_slug():
    assert get_cached_slug() == "f67108fc906341d8b15209ce88ebc3d2"
    reset_notebook_slug()
    assert get_cached_slug() == None



import os
from unittest import TestCase, mock
from textwrap import dedent

import yaml
import pytest

from jovian.utils.envfile import (check_error, check_pip_failed, dump_environment_to_yaml_file, extract_env_name,
                                  extract_env_packages, extract_package_from_line, extract_pip_packages,
                                  get_environment_dict, identify_env_file, remove_packages, request_env_name,
                                  sanitize_envfile, serialize_packages, write_env_name)
from jovian.tests.resources.shared import temp_directory, fake_envfile


def test_identify_env_file_no_environment_file():
    with temp_directory():
        assert identify_env_file(env_fname=None) == None


def test_identify_env_file_exists():
    with fake_envfile():
        assert identify_env_file(env_fname=None) == 'environment.yml'


def test_get_environment_dict():
    with fake_envfile():
        expected_result = {'prefix': '/home/admin/anaconda3/envs/test-env',
                           'dependencies': ['mixpanel=1.11.0', 'sigmasix=1.91.0', 'sqlite',
                                            {'pip': ['six==1.11.0', 'sqlite==2.0.0']}],
                           'channels': ['defaults'], 'name': 'test-env'}
        environment = get_environment_dict(env_fname='environment.yml')

        assert environment == expected_result


def test_get_environment_dict_non_existent_file():
    with fake_envfile():
        expected = {'prefix': '/home/admin/anaconda3/envs/test-env',
                    'dependencies': ['mixpanel=1.11.0', 'sigmasix=1.91.0', 'sqlite',
                                     {'pip': ['six==1.11.0', 'sqlite==2.0.0']}],
                    'channels': ['defaults'], 'name': 'test-env'}
        environment = get_environment_dict(env_fname='environment.yml')

        with pytest.raises(FileNotFoundError):
            get_environment_dict(env_fname='non-existent-file.yml')


def test_get_environment_dict_raises_exception_invalid_file():
    with fake_envfile():
        with open('invalid-yaml-file.yml', 'w') as f:
            f.write(dedent("""
            ok:
                sub1: sub1
                ][
            hello:
                world: world
                some: other
            """))
        assert get_environment_dict('invalid-yaml-file.yml') == {}


@pytest.mark.parametrize(
    "func, env_fname, expected_result",
    [
        (extract_env_name, 'environment.yml', 'test-env'),
        (extract_env_name, 'empty-yaml-file.yml', None),
        (extract_env_packages, 'environment.yml', ['mixpanel=1.11.0', 'sigmasix=1.91.0', 'sqlite',
                                                   'six==1.11.0', 'sqlite==2.0.0']),
        (extract_env_packages, 'empty-yaml-file.yml', []),
        (extract_pip_packages, 'environment.yml', ['six==1.11.0', 'sqlite==2.0.0']),
        (extract_pip_packages, 'empty-yaml-file.yml', []),
    ]
)
def test_extract(func, env_fname, expected_result):
    with fake_envfile():
        assert func(env_fname) == expected_result

        with pytest.raises(FileNotFoundError):
            func('non-existent-file.yml')


@pytest.mark.parametrize(
    "error_str, expected_result",
    [
        (
            '''ResolvePackageNotFound:
                    - mixpanel=1.11.0''',
            ('unresolved', ['mixpanel=1.11.0'])
        ),
        (
            '''UnsatisfiableError:
                            - sigmasix=1.91.0''',
            ('unsatisfiable', ['sigmasix=1.91.0'])
        ),
        (
            '''AnyOtherException:
                            - sigmasix=1.91.0''',
            (None, [])
        ),
    ]
)
def test_check_error(error_str, expected_result):
    with fake_envfile():
        packages = extract_env_packages(env_fname='environment.yml')
        assert check_error(error_str=error_str, packages=packages) == expected_result


def test_check_error_no_packages():
    error_str = '''ResolvePackageNotFound:
                    - mixpanel=1.11.0'''
    packages = []
    assert check_error(error_str=error_str, packages=[]) == ('unresolved', [])


@pytest.mark.parametrize(
    "line, expected_result",
    [
        ('- mixpanel=1.11.0', 'mixpanel=1.11.0'),
        ('sqlite<3x not compatible', 'sqlite'),
        ('', None),
        ('line containing six < 3.0.x', 'six==1.11.0'),
    ]
)
def test_extract_package_from_line(line, expected_result):
    with fake_envfile():
        packages = extract_env_packages(env_fname='environment.yml')
        assert extract_package_from_line(line=line, packages=packages) == expected_result


def test_dump_environment_to_yaml_file_normal():
    with fake_envfile():
        data = {'channels': ['defaults'],
                'dependencies': ['mixpanel=1.11.0',
                                 'sigmasix=1.91.0',
                                 'sqlite',
                                 {'pip': ['six==1.11.0', 'sqlite==2.0.0']}],
                'name': 'test-env',
                'prefix': '/home/admin/anaconda3/envs/test-env'}
        dump_environment_to_yaml_file('test.yml', data)

        assert get_environment_dict('test.yml') == data


@pytest.mark.parametrize(
    "data, name",
    [
        (
            {'channels': ['defaults'],
                'dependencies': ['mixpanel=1.11.0'],
                'name': 'test-env',
                'prefix': '/home/admin/anaconda3/envs/test-env'},
            'test-env-changed'
        ),
        (
            {'channels': ['defaults'],
                'dependencies': ['mixpanel=1.11.0'],
                'prefix': '/home/admin/anaconda3/envs/test-env'},
            'test-env'
        ),
        (
            {'channels': ['defaults'],
                'dependencies': ['mixpanel=1.11.0'],
                'name': 'test-env',
                'prefix': '/home/admin/anaconda3/envs/test-env'},
            'test-env'
        ),
    ]
)
def test_write_env_name(data, name):
    with fake_envfile():
        dump_environment_to_yaml_file('test.yml', data)

        write_env_name(name, 'test.yml')

        assert get_environment_dict('test.yml')['name'] == name


@pytest.mark.parametrize(
    "dependencies, expected_result",
    [
        (['dep1', 'dep2', 'mixpanel=1.11.0', 'sqlite'], ['dep1', 'dep2']),
        (['dep1', 'mixpanel=1.11.0', {'pip': ['six==1.11.0', 'dep3']}], ['dep1', {'pip': ['dep3']}]),
    ]
)
def test_remove_packages(dependencies, expected_result):
    with fake_envfile():
        packages = extract_env_packages(env_fname='environment.yml')
        assert remove_packages(dependencies, packages) == expected_result


def test_sanitize_envfile():
    with fake_envfile():
        packages = ['six==1.11.0', 'sigmasix=1.91.0']
        sanitize_envfile('environment.yml', packages)

        expected_result = {'channels': ['defaults'],
                           'dependencies': ['mixpanel=1.11.0',
                                            'sqlite',
                                            {'pip': ['sqlite==2.0.0']}],
                           'name': 'test-env',
                           'prefix': '/home/admin/anaconda3/envs/test-env'}

        assert get_environment_dict('environment.yml') == expected_result


@pytest.mark.parametrize(
    "dependencies, expected_result",
    [
        (
            ['dep1', 'mixpanel=1.11.0', {'pip': None}],
            ['dep1', 'mixpanel=1.11.0']
        ),
        (
            ['dep1', 'mixpanel=1.11.0', {'pip': ['six==1.11.0', 'dep3']}],
            ['dep1', 'mixpanel=1.11.0', 'six==1.11.0', 'dep3']
        ),
        (
            ['dep1', 'mixpanel=1.11.0', 'sigmasix=1.91.0', {'pip': ['six==1.11.0', 'dep3']}],
            ['dep1', 'mixpanel=1.11.0', 'sigmasix=1.91.0', 'six==1.11.0', 'dep3']
        ),
    ]
)
def test_serialize_packages(dependencies, expected_result):
    assert serialize_packages(dependencies) == expected_result


@pytest.mark.parametrize(
    "prompt_ret_val, extract_env_name_ret_val, expected_result",
    [
        (
            "",
            "test-env",
            "test-env",
        ),
        (
            "test-env-changed",
            "test-env",
            "test-env-changed",
        ),
        (
            "test-env-changed",
            None,
            "test-env-changed",
        ),
        (
            "",
            None,
            "base",
        ),
    ]
)
def test_request_env_name(prompt_ret_val, extract_env_name_ret_val, expected_result):
    with fake_envfile():
        with mock.patch("jovian.utils.envfile.click.prompt", return_value=prompt_ret_val), \
                mock.patch("jovian.utils.envfile.extract_env_name", return_value=extract_env_name_ret_val):
            assert request_env_name(env_name=None, env_fname='environment.yml') == expected_result


@pytest.mark.parametrize(
    "error_str, expected_result",
    [
        ('''Could not install
                    Pip failed with error code 2''', True),
        ('''Pip successfully installed package''', False),
    ]
)
def test_check_pip_failed(error_str, expected_result):
    assert check_pip_failed(error_str) == expected_result



import pytest
from textwrap import dedent
from unittest import mock, TestCase
from unittest.mock import ANY
import os
from jovian.utils.environment import (get_conda_bin, get_conda_env_name, read_conda_env,
                                      upload_conda_env, print_conda_message, read_pip_env, upload_pip_env)
from jovian.utils.error import CondaError
from jovian.tests.resources.shared import temp_directory

pip_env = dedent("""
            alabaster==0.7.12
            apipkg==1.5
            appdirs==1.4.3
            appnope==0.1.0
            argh==0.26.2
            astroid==2.2.5
            attrs==19.1.0
            autopep8==1.5
            Babel==2.7.0
        """)

ENV_STR = dedent("""
            name: jovian-py-dev
            channels:
            - defaults
            dependencies:
            - appnope=0.1.0
            - blas=1.0
            prefix: /Users/rohitsanjay/miniconda3/envs/jovian-py-dev
        """).strip()


@mock.patch("os.popen")
def test_get_conda_bin(mock_popen):
    ret1 = mock.Mock()
    ret1.read().strip.return_value = 'usage: conda [-h] [-V] command ...'
    mock_popen.side_effect = [ret1]
    assert get_conda_bin() == 'conda'


@mock.patch.dict("os.environ", {"CONDA_EXE": "/Users/username/miniconda3/bin/conda"})
@mock.patch("os.popen")
def test_get_conda_bin_look_for_conda_exe(mock_popen):
    ret1, ret2, ret3 = mock.Mock(), mock.Mock(), mock.Mock()
    ret1.read().strip.return_value = ''
    ret2.read().strip.return_value = 'usage: conda [-h] [-V] command ...'
    mock_popen.side_effect = [ret1, ret2]

    assert get_conda_bin() == "/Users/username/miniconda3/bin/conda"


@mock.patch.dict("os.environ", "")
@mock.patch("os.popen")
def test_get_conda_bin_look_for_conda_exe_raises_error(mock_popen):
    ret1, ret2 = mock.Mock(), mock.Mock()
    ret1.read().strip.return_value = ''
    mock_popen.side_effect = [ret1, ret1]

    with pytest.raises(CondaError):
        get_conda_bin()


@pytest.mark.parametrize(
    "env_dict, expected_result",
    [
        ({"CONDA_DEFAULT_ENV": "fake-env"}, "fake-env"),
        ({"CONDA_DEFAULT_ENV": ""}, "base"),
        ({}, "base")
    ]
)
def test_get_conda_env_name(env_dict, expected_result):
    with mock.patch.dict("os.environ", env_dict, clear=True):
        assert get_conda_env_name() == expected_result


@mock.patch("jovian.utils.environment.get_conda_env_name", return_value='fake-env')
@mock.patch("jovian.utils.environment.get_conda_bin", return_value="/usr/bin/conda")
@mock.patch("os.popen")
def test_read_conda_env_name_none(mock_popen, mock_get_conda_bin, mock_get_conda_env_name):
    mock_popen().read.return_value = ENV_STR
    assert read_conda_env() == ENV_STR


@mock.patch("jovian.utils.environment.get_conda_env_name", return_value='no-env')
@mock.patch("jovian.utils.environment.get_conda_bin", return_value="/usr/bin/conda")
@mock.patch("os.popen")
def test_read_conda_env_empty_env_str_raises_error(mock_popen, mock_get_conda_bin, mock_get_conda_env_name):
    mock_popen().read.return_value = ''
    with pytest.raises(CondaError):
        read_conda_env()


@mock.patch("jovian.utils.environment.read_conda_env")
@mock.patch("jovian.utils.environment.get_conda_env_name")
@mock.patch("jovian.utils.environment.get_platform", return_value="macos")
@mock.patch("jovian.utils.environment.upload_file")
def test_upload_conda_env(mock_upload_file, mock_get_platform, mock_get_conda_env_name, mock_read_conda_env):
    with temp_directory():
        mock_read_conda_env.return_value = ENV_STR

        with open('environment-linux.yml', 'w') as f:
            f.write(ENV_STR)

        upload_conda_env('fake_gist_slug', version=2)

        calls = [
            mock.call(
                gist_slug='fake_gist_slug',
                file=('environment.yml', ENV_STR),
                version=2),
            mock.call(
                gist_slug='fake_gist_slug',
                file=('environment-linux.yml', ANY),
                version=2),
            mock.call(
                gist_slug='fake_gist_slug',
                file=('environment-macos.yml', ENV_STR),
                version=2)
        ]

        mock_upload_file.assert_has_calls(calls)


def test_print_conda_message(capsys):
    print_conda_message('fake-env')

    expected_result = dedent("""
                        [jovian] 
                        #
                        # To activate this environment, use
                        #
                        #     $ conda activate fake-env
                        #
                        # To deactivate an active environment, use
                        #
                        #     $ conda deactivate
                    """).strip()
    captured = capsys.readouterr()
    assert captured.out.strip() == expected_result.strip()


@mock.patch("os.popen")
def test_read_pip_env(mock_popen):
    mock_popen().read.return_value = pip_env
    assert read_pip_env() == pip_env


@mock.patch("os.popen")
def test_read_pip_env_raise_error(mock_popen):
    mock_popen().read.return_value = ''
    with pytest.raises(Exception):
        read_pip_env()


@mock.patch("jovian.utils.environment.upload_file")
@mock.patch("os.popen")
def test_upload_pip_env(mock_popen, mock_upload_file):
    mock_popen().read.return_value = pip_env
    upload_pip_env('fake_gist_slug', version=2)

    mock_upload_file.assert_called_with(gist_slug='fake_gist_slug', file=('requirements.txt', pip_env), version=2)



import json
from unittest import TestCase, mock

import pytest

from jovian.tests.resources.shared import MockResponse
from jovian.utils.jupyter import (
    get_notebook_server_path, has_ipynb_shell, in_notebook, get_notebook_path_py, get_notebook_path, set_notebook_name,
    get_notebook_name_saved, get_notebook_name, get_notebook_history, save_notebook)


@pytest.mark.parametrize(
    "get_ipython, expected_result",
    [
        ('ZMQInteractiveShell', True),
        ('TerminalInteractiveShell', False)
    ]
)
@mock.patch("IPython.get_ipython")
def test_has_ipynb_shell(mock_get_ipython, get_ipython, expected_result):
    mock_get_ipython().__class__.__name__ = get_ipython
    assert has_ipynb_shell() == expected_result


@mock.patch('builtins.__import__', return_value=ImportError('fake import error'))
@mock.patch("IPython.get_ipython")
def test_has_ipynb_shell_except_return_false(mock_get_ipython, mock_import):
    assert has_ipynb_shell() == False


@mock.patch("ipykernel.kernelapp.IPKernelApp")
@mock.patch("IPython.get_ipython")
def test_in_notebook_false(mock_get_ipython, mock_IPKernelApp):
    mock_get_ipython().__class__.__name__ = 'ZMQInteractiveShell'
    mock_IPKernelApp.initialized.return_value = False
    assert in_notebook() == False

    mock_IPKernelApp.initialized.return_value = True
    assert in_notebook() == True


def mock_list_running_servers():
    return [{'base_url': '/', 'hostname': 'localhost', 'notebook_dir': '/Users/rohitsanjay', 'password': False, 'pid': 20604,
             'port': 8888, 'secure': False, 'token': '87b9edfc60e39cda411857a364d3948b999e9cbec22b4f58',
             'url': 'http://localhost:8888/'}]


def mock_request_get(*args, **kwargs):
    data = [
        {
            "id": "6492b857-8ea0-45af-a1ab-8769dd066235",
            "path": "Untitled.ipynb",
            "name": "",
            "type": "notebook",
            "kernel": {
                "id": "ee27a181-852b-4f21-a2c0-a34eacef769c",
                "name": "python3",
                "last_activity": "2020-03-02T08:00:15.767476Z",
                "execution_state": "idle",
                "connections": 1},
            "notebook": {
                "path": "Fake-Notebook.ipynb",
                "name": ""
            }
        }
    ]

    return MockResponse(data, 200, json.dumps(data))


def mock_re():
    ret1 = mock.Mock()
    ret1().group.return_value = "ee27a181-852b-4f21-a2c0-a34eacef769c"
    return ret1


@pytest.mark.parametrize(
    "func, expected_result",
    [
        (get_notebook_server_path, 'Fake-Notebook.ipynb'),
        (get_notebook_path_py, '/Users/rohitsanjay/Fake-Notebook.ipynb'),
        (get_notebook_path, '/Users/rohitsanjay/Fake-Notebook.ipynb'),
    ]
)
@mock.patch("ipykernel.connect.get_connection_file", return_value='')
@mock.patch("re.search", side_effect=mock_re())
@mock.patch("notebook.notebookapp.list_running_servers", return_value=mock_list_running_servers())
@mock.patch("requests.get", side_effect=mock_request_get)
def test_notebook_path_funs(mock_get, mock_servers, mock_re, mock_get_connection_file, func, expected_result):
    assert func() == expected_result


@mock.patch("jovian.utils.jupyter.get_notebook_name_saved", return_value="Fake-Notebook.ipynb")
@mock.patch("os.getcwd", return_value="/Users/rohitsanjay")
def test_get_notebook_path_javascript(mock_getcwd, mock_get_notebook_name_saved):
    assert get_notebook_path() == '/Users/rohitsanjay/Fake-Notebook.ipynb'


@mock.patch('IPython.get_ipython')
@mock.patch("jovian.utils.jupyter.in_notebook", return_value=True)
def test_set_notebook_name(mock_in_notebook, mock_get_ipython):
    set_notebook_name()

    mock_get_ipython().run_cell_magic.assert_called_with('javascript', '',
                                                         "if (window.IPython && IPython.notebook.kernel) IPython.notebook.kernel.execute('jovian.utils.jupyter.get_notebook_name_saved = lambda: \"' + IPython.notebook.notebook_name + '\"')")


def test_get_notebook_name_saved():
    assert get_notebook_name_saved() == None


@mock.patch("jovian.utils.jupyter.get_notebook_path", side_effect=['/Users/rohitsanjay/Fake-Notebook.ipynb', None])
def test_get_notebook(mock_get_notebook_path):
    assert get_notebook_name() == 'Fake-Notebook.ipynb'

    assert get_notebook_name() == None


@mock.patch('IPython.get_ipython')
def test_get_notebook_history(mock_get_ipython):
    get_notebook_history()

    mock_get_ipython().magic.assert_called_with('%history')


@mock.patch('IPython.get_ipython')
@mock.patch("jovian.utils.jupyter.in_notebook", return_value=True)
def test_save_notebook(mock_in_notebook, mock_get_ipython):
    save_notebook()

    mock_get_ipython().run_cell_magic.assert_called_with(
        'javascript',
        '',
        'window.require && require(["base/js/namespace"],function(Jupyter){Jupyter.notebook.save_checkpoint()})')



from textwrap import dedent
from unittest import TestCase, mock

import pytest
from jovian._version import __version__
from jovian.tests.resources.shared import MockResponse
from jovian.utils.latest import _get_latest_version, _print_update_message, check_update
from pkg_resources import parse_version


@mock.patch("requests.get")
def test_get_latest_version_normal(mock_get):
    mock_get().json.return_value = {'info': {'version': '0.2.0.dev1'}}

    expected_result = "0.2.0.dev1"
    assert _get_latest_version() == expected_result


@mock.patch("requests.get", return_value=mock.Mock(return_value=ConnectionError('fake requests.get error')))
def test_get_latest_version_exception(mock_get):
    assert _get_latest_version() == __version__


@pytest.mark.parametrize(
    "in_notebook, expected_result",
    [
        (
            True,
            dedent("""
            [jovian] Update Available: 0.0.1 --> 0.0.2
            [jovian] Run `!pip install jovian --upgrade` to upgrade
            """).strip()
        ),
        (
            False,
            dedent("""
            [jovian] Update Available: 0.0.1 --> 0.0.2
            [jovian] Run `pip install jovian --upgrade` to upgrade
            """).strip()
        ),
    ]
)
def test_print_update_message(in_notebook, expected_result, capsys):
    with mock.patch("jovian.utils.latest.in_notebook", return_value=in_notebook):
        _print_update_message('0.0.1', '0.0.2')
        captured = capsys.readouterr()
        assert captured.out.strip() == expected_result


@pytest.mark.parametrize(
    "probability, expected_result, random_val",
    [
        (1, dedent("""
            [jovian] Update Available: 0.0.1 --> 0.0.2
            [jovian] Run `!pip install jovian --upgrade` to upgrade
            """).strip(), 0.3),
        (None, dedent("""
            [jovian] Update Available: 0.0.1 --> 0.0.2
            [jovian] Run `!pip install jovian --upgrade` to upgrade
            """).strip(), 0.4),
        (None, "", 0.9),
    ]
)
@mock.patch("jovian.utils.latest._get_latest_version", mock.Mock(return_value='0.0.2'))
@mock.patch("jovian.utils.latest.in_notebook", mock.Mock(return_value=True))
def test_check_update(probability, expected_result, random_val, capsys):
    with mock.patch("jovian.utils.latest.random", return_value=random_val):
        import jovian
        jovian.utils.latest.__version__ = '0.0.1'

        if probability:
            check_update(probability)
        else:
            check_update()

        captured = capsys.readouterr()
        assert captured.out.strip() == expected_result
